package com.bocloud.rms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.rms.intf.BillReportDownloadService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * @author tyl
 */
@Api(value = "报表", tags = "报表")
@RestController
@RequestMapping("/v1/reports")
public class BillReportDownloadController {
    private final BillReportDownloadService billReportDownloadService;

    public BillReportDownloadController(BillReportDownloadService billReportDownloadService) {
        this.billReportDownloadService = billReportDownloadService;
    }


    /**
     * 租户账单报表导出
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户账单表报下载", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/tenant/download")
    public Result tenantDownload(Pager pager,
                                 @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                                 HttpServletResponse response) {
        BocloudResult<Map<String, Object>> bocloudResult = billReportDownloadService.billReportDownload(pager, context, "tenant");
        if (bocloudResult.isSuccess()) {
            String content;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject
                        .parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new Result(false, "内部错误");
            }
        }
        return bocloudResult;
    }

    /**
     * 运营账单报表导出
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "运营账单表报下载", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/operation/download")
    public Result operationDownload(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                                    HttpServletResponse response) {
        BocloudResult<Map<String, Object>> bocloudResult = billReportDownloadService.billReportDownload(pager, context, "operation");
        if (bocloudResult.isSuccess()) {
            String content;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject
                        .parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new Result(false, "内部错误");
            }
        }
        return bocloudResult;
    }
}
