package com.bocloud.rms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.rms.intf.OperationReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by liuyuanyuan on 2020/7/10 18:15
 */
@Api(value = "运营报表", tags = "运营报表")
@RestController
@RequestMapping("/v1/operations/reports")
public class OperationReportController {
    @Autowired
    private OperationReportService operationReportService;

    @ApiOperation(tags = {"CMC", "CSC"}, value = "资源图标统计", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult resourceData(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                      @RequestParam(value = "vdcId", required = false) Long vdcId,
                                      @RequestParam(value = "clusterId", required = false) Long clusterId,
                                      @RequestParam(value = "name", required = false) String name,
                                      @RequestParam(value = "category", required = false) String category,
                                      @RequestParam(value = "tenantId", required = false) Long tenantId,
                                      @RequestParam(value = "vendorType", required = false) String vendorType,
                                      @RequestParam(value = "time", required = false) String time,
                                      @RequestParam(value = "startTime", required = false) String startTime,
                                      @RequestParam(value = "endTime", required = false) String endTime) {
        return operationReportService.resourceData(vendorId, vdcId, clusterId, name, category, tenantId, vendorType, time, startTime, endTime);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "监控告警报表统计", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/monitor")
    public BocloudResult monitorData(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                     @RequestParam(value = "name", required = false) String name,
                                     @RequestParam(value = "category", required = false) String category,
                                     @RequestParam(value = "type", required = false) String type,
                                     @RequestParam(value = "limit", required = false) Integer limit,
                                     @RequestParam(value = "time", required = false) String time) {
        return operationReportService.monitorData(vendorId, name, category, type, limit, time);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户资源统计", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/tenants")
    public BocloudResult tenantData(Pager pager, @RequestParam(value = Common.CONDITION, required = false) String condition,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return operationReportService.tenantData(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(),
                pager.getSimple(), condition, context);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "计算资源统计", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/computers")
    public BocloudResult computerData(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                      @RequestParam(value = "vdcId", required = false) Long vdcId,
                                      @RequestParam(value = "clusterId", required = false) Long clusterId,
                                      @RequestParam(value = "name", required = false) String name,
                                      @RequestParam(value = "vendorType", required = false) String vendorType,
                                      @RequestParam(value = "paging", required = false) String paging) {
        return operationReportService.computerData(vendorId, vdcId, clusterId, name, vendorType, JSONObject.parseObject(paging));
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "运营资源报表导出", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/download")
    public BocloudResult download(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                  @RequestParam(value = "vdcId", required = false) Long vdcId,
                                  @RequestParam(value = "clusterId", required = false) Long clusterId,
                                  @RequestParam(value = "id", required = false) Long id,
                                  @RequestParam(value = "name", required = false) String name,
                                  @RequestParam(value = "category", required = false) String category,
                                  @RequestParam(value = "vendorType", required = false) String vendorType,
                                  @RequestParam(value = "time", required = false) String time,
                                  HttpServletRequest request, HttpServletResponse response) {
        BocloudResult bocloudResult = operationReportService.download(vendorId, vdcId, clusterId, id, name, category, vendorType, time);
        if (bocloudResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject
                        .parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new BocloudResult(false, "内部错误");
            }
        }
        return bocloudResult;
    }

}
