package com.bocloud.rms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.rms.common.enums.PcType;
import com.bocloud.rms.intf.PublicCloudBillService;
import com.bocloud.rms.intf.ReportService;
import com.bocloud.rms.service.utils.CsvExportUtil;
import com.bocloud.service.core.Service;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.bocloud.rms.enums.CloudProviderType.HUAWEI;

/**
 * 公有云账单
 *
 * @author lyy
 * @version 1.0
 * @date 2021/2/2 16:22
 */
@RestController
@RequestMapping("/v1/pc/bills")
@Api(value = "公有云账单", tags = "公有云账单")
public class PublicCloudBillController {
    private static Logger logger = LoggerFactory.getLogger(PublicCloudBillController.class);
    @Autowired
    private PublicCloudBillService publicCloudBillService;
    @Autowired
    private Service service;

    /**
     * 公有云账单导入
     *
     * @param request
     * @param response
     * @param context
     * @return
     * @throws IOException
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "导入", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/upload", method = {RequestMethod.POST})
    public BocloudResult upload(HttpServletRequest request, HttpServletResponse response,
                                @RequestParam(value = "year", required = true) Long year,
                                @RequestParam(value = "month", required = true) Long month,
                                @RequestParam(value = "source", required = true) String source,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            MultipartHttpServletRequest multipartHttpServletRequest = (MultipartHttpServletRequest) request;
            MultiValueMap<String, MultipartFile> multiFileMap = multipartHttpServletRequest.getMultiFileMap();
            MultipartFile item = null;
            if (multiFileMap.values().size() > 0) {
                List<MultipartFile> multiFileList = multiFileMap.values().iterator().next();
                item = multiFileList.get(0);
            }
            JSONObject object = new JSONObject();
            object.put("year", year);
            object.put("month", month);
            object.put("source", source);
//            return publicCloudBillService.upload(item, context.getTarget(), service.getInstance(), object.toJSONString());
            String instance = service.getHost() + ":" + service.getPort();
            return publicCloudBillService.upload(item, context.getTarget(), instance, object.toJSONString());

        } catch (Exception e) {
            logger.info("导入失败", e);
        }
        return new BocloudResult(false, "文件错误");
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "下载公有云账单列表模板", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(@RequestParam(value = "type") String type, HttpServletRequest request, HttpServletResponse response,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {

        try {
            if (null == type) {
                throw new Exception("请选择正确的公有云模板，再进行模板下载");
            }
            String fileName = "";
            String[] title = new String[0];
            String titles = "";
            switch (PcType.getByType(type)) {
                case ALI:
                    fileName += PcType.ALI.getPcName();
                    titles = "账期,账号ID,账号,Owner账号ID,Owner账号,产品Code,产品,产品明细Code,产品明细,业务类型,消费类型,消费时间," +
                            "账单开始时间,账单结束时间,订单号/账单号,账单类型,原价,优惠金额,优惠券抵扣,抹零金额,应付金额,现金支付,代金券抵扣,储值卡支付金额,欠费金额,支付状态,币种";
                    break;
                case HUAWEI:
                    fileName += PcType.HUAWEI.getPcName();
                    titles = "账期,账号,产品类型,产品,计费模式,账单类型,官网价(¥),优惠金额(¥),抹零金额(¥),应付金额(¥),现金支付(¥)," +
                            "信用额度支付(¥),代金券抵扣(¥),现金券抵扣(¥),储值卡抵扣(¥),欠费金额(¥)";
                    break;
                default:
                    break;
            }
//            fileName +=  "账单导入模板" + System.currentTimeMillis() + ".csv";
            fileName += "账单导入模板";
            CsvExportUtil.responseSetProperties(fileName, response);
            CsvExportUtil.doTitle(titles, response.getOutputStream());

        } catch (Exception ioException) {
            logger.info("context", ioException);
        }
    }

    /**
     * 公有云账单下载
     *
     * @param id
     * @param context
     * @param request
     * @param response
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单下载", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/{id}/download")
    public BocloudResult tenantDownload(@PathVariable(value = Common.ID) Long id, Pager pager,
                                        @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                                        HttpServletResponse response) {
        BocloudResult bocloudResult = publicCloudBillService.download(id, pager, context);
        if (bocloudResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject
                        .parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("text/csv;charset=utf-8");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
                logger.info("下载共有云账单成功，文件名称为：" + result.get("fileName").toString());
            } catch (Exception e) {
                logger.error("内部错误，请检查");
//                return new BocloudResult(false, "内部错误");
            }
        }
        return null;
    }

    /**
     * 公有云账单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return publicCloudBillService.templates(pager, context);
    }

    /**
     * 删除公有云账单
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除公有云账单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return publicCloudBillService.removeTemplate(id, context);
    }

    /**
     * 账单详情
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult detail(Pager pager,
                                @PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return publicCloudBillService.detail(id, pager, context);
    }

    /**
     * @param id
     * @param context
     * @return com.bocloud.common.model.BocloudResult
     * @Author: lxy
     * @Description: 账单详情选择条件
     * @Date 2021/7/22 14:45
     * @Param pager
     * @throws:
     **/
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单详情选择条件", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/select")
    public BocloudResult detailSelect(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return publicCloudBillService.detailSelect(id, context);
    }

    /**
     * 公有云账单列数据变更
     *
     * @param parmas
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单列数据变更", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/columns")
    public BocloudResult detail(@RequestBody JSONObject parmas,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != parmas) {

            return publicCloudBillService.columns(parmas.getLong("propertyId"), parmas.getBoolean("checked"), context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 公有云账单合并
     *
     * @param combine
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "公有云账单合并", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/combine")
    public BocloudResult combine(@RequestBody List<Long> combine,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != combine && 0 != combine.size()) {
            return publicCloudBillService.combine(combine, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

}