package com.bocloud.rms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.rms.intf.ReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * 报表
 *
 * @author lyy
 * @version 1.0
 * @date 2021/3/10 17:44
 */
@Api(value = "报表", tags = "报表")
@RestController
@RequestMapping("/v1/reports")
public class ReportController {
    @Autowired
    private ReportService reportService;

    /**
     * 报表虚拟机清单
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "报表虚拟机清单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/vms")
    public BocloudResult resources(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return reportService.vmList(pager, context);
    }

    /**
     * 报表虚拟机清单下载
     *
     * @param pager
     * @param context
     * @param request
     * @param response
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "报表虚拟机清单下载", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/vms/download")
    public BocloudResult vmsDownload(Pager pager,
                                     @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                                     HttpServletResponse response) {
        BocloudResult bocloudResult = reportService.vmsDownload(pager, context);
        if (bocloudResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject
                        .parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("text/csv");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), StandardCharsets.ISO_8859_1);
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new BocloudResult(false, "内部错误");
            }
        }
        return bocloudResult;
    }

}
