package com.bocloud.rms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.entity.OrderItem;
import com.bocloud.rms.enums.OrderItemStatus;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.google.common.collect.Maps;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * 订单明细数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("orderItemRepository")
public class OrderItemRepository extends BasicGenericDao<OrderItem, Long> {

    public OrderItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<OrderItem> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.* from order_item a where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(OrderItem.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from order_item a where 1 = 1  ";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "update order_item set is_deleted = 1 where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<OrderItem> queryTenantResource(List<Long> orderIds) {
        String sql =
                "SELECT a.*,b.tenant_id,b.vendor_id FROM order_item a LEFT JOIN service_order b ON a.order_id = b.id WHERE a.is_deleted = 0 AND a.status IN('OpenedSuccess','ChangeSuccess','Changing','Unsubscribing','ExtensionSuccess','ExtensionFailure','Extensioning') and b.is_deleted = 0  ";
        Map<String, Object> paramMap = new HashMap<>();
        if (!orderIds.isEmpty()) {
            sql += " AND a.order_id IN(:orderIds) ";
            paramMap.put("orderIds", orderIds);
        }
        return this.list(OrderItem.class, sql, paramMap);
    }

    public List<OrderItem> resourceCount(List<Long> tenantIds, List<String> codes, Long vendorId) {
        String sql =
                "SELECT a.*,b.tenant_id,b.vendor_id FROM order_item a LEFT JOIN service_order b ON a.order_id = b.id WHERE a.is_deleted = 0 AND a.status IN('OpenedSuccess','ChangeSuccess','Changing','Unsubscribing','ExtensionSuccess','ExtensionFailure','Extensioning') and a.effective_time is NOT NULL ";
        Map<String, Object> paramMap = Maps.newHashMap();
        if (null != codes) {
            sql += " AND a.service_code in(:codes) ";
            paramMap.put("codes", codes);
        }
        if (!tenantIds.isEmpty()) {
            sql += " and b.tenant_id in(:tenantIds) ";
            paramMap.put("tenantIds", tenantIds);
        }
        if (null != vendorId) {
            sql += " and b.vendor_id = :vendorId ";
            paramMap.put("vendorId", vendorId);
        }
        return this.list(OrderItem.class, sql, paramMap);
    }

}
