package com.bocloud.rms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.entity.PcBillTemplate;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @version 1.0
 * @date 2021/2/22 10:21
 */
@Repository("pcBillTemplateRepository")
public class PcBillTemplateRepository extends BasicGenericDao<PcBillTemplate, Long> {
    public PcBillTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<PcBillTemplate> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.*,b.name AS creator_name from pc_bill_template a LEFT JOIN users b ON a.creator_id = b.id where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(PcBillTemplate.class, sql, paramMap);
    }

    public PcBillTemplate query(Long id) {
        String sql = "select * from pc_bill_template where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<PcBillTemplate> templates = this.list(PcBillTemplate.class, sql, params);
        return templates.isEmpty() ? null : templates.get(0);
    }

    public PcBillTemplate queryByName(String name, String source) {
        String sql = "select * from pc_bill_template where name = :name and source = :source and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.NAME, name);
        params.put("source", source);
        List<PcBillTemplate> templates = this.list(PcBillTemplate.class, sql, params);
        return templates.isEmpty() ? null : templates.get(0);
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update pc_bill_template set is_deleted = 1, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
//        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from pc_bill_template where is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }
}
