package com.bocloud.rms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.entity.PcTemplateProperty;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @version 1.0
 * @date 2021/2/22 10:31
 */
@Repository("pcTemplatePropertyRepository")
public class PcTemplatePropertyRepository extends BasicGenericDao<PcTemplateProperty, Long> {
    public PcTemplatePropertyRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<PcTemplateProperty> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select * from pc_template_property where is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(PcTemplateProperty.class, sql, paramMap);
    }

    public PcTemplateProperty query(Long id) {
        String sql = "select * from pc_template_property where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<PcTemplateProperty> properties = this.list(PcTemplateProperty.class, sql, params);
        return properties.isEmpty() ? null : properties.get(0);
    }

    public List<PcTemplateProperty> queryByTemplateId(Long templateId) {
        String sql = "select * from pc_template_property where template_id = :templateId";
        Map<String, Object> params = MapTools.simpleMap("templateId", templateId);
        return this.list(PcTemplateProperty.class, sql, params);
    }

    public boolean deleteByTemplateId(Long templateId, Long userId) {
        String sql = "update pc_template_property set is_deleted = 1, mender_id = :menderId where template_id = :templateId";
        Map<String, Object> params = MapTools.simpleMap("templateId", templateId);
        params.put("menderId", userId);
//        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from pc_template_property where is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<PcTemplateProperty> queryByPropertyAll(Long templateId) {
        String sql = "SELECT * FROM pc_template_property WHERE is_deleted = 0 AND template_id = :templateId ";
        Map<String, Object> paramMap = MapTools.simpleMap("templateId", templateId);
        return this.list(PcTemplateProperty.class, sql, paramMap);
    }

    public List<PcTemplateProperty> queryByPropertyStatus(Long templateId) {
        String sql = "SELECT * FROM pc_template_property WHERE is_deleted = 0 AND is_show =1 AND template_id = :templateId ";
        Map<String, Object> paramMap = MapTools.simpleMap("templateId", templateId);
        return this.list(PcTemplateProperty.class, sql, paramMap);
    }

    public List<PcTemplateProperty> query(Long templateId, List<String> names) {
        String sql = "SELECT * FROM pc_template_property WHERE is_deleted = 0 AND template_id = :templateId and name in(:names) ";
        Map<String, Object> paramMap = MapTools.simpleMap("templateId", templateId);
        paramMap.put("names", names);
        return this.list(PcTemplateProperty.class, sql, paramMap);
    }

}
