package com.bocloud.rms.repository;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.rms.internel.entity.CloudServer;
import com.bocloud.rms.internel.entity.User;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 报表数据查询
 *
 * @author lyy
 * @version 1.0
 * @date 2021/3/11 16:45
 */
@Repository("reportRepository")
public class ReportRepository extends BasicGenericDao {

    public ReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public Map<Long, User> userList() {
        //缓存所有的用户信息
        String userSql = "SELECT * FROM users WHERE is_deleted = 0";
        List<Object> userList = this.list(User.class,userSql);
        Map<Long, User> userMap = Maps.newHashMap();
        for (Object object : userList) {
            User user = (User) object;
            userMap.put(user.getId(), user);
        }
        return userMap;
    }

    public List<CloudServer> vmList(int page, int rows, List<Param> params, Map<String, String> sorter) {
        Map<Long, User> userMap = userList();
        //虚拟机信息查询
        StringBuilder sql = new StringBuilder("SELECT a.*,d.name AS target_name, b.name AS tenant_name, c.name AS project_name, e.name AS vendor_name, f.name AS owner_name FROM cloud_server a LEFT JOIN tenant b ON a.tenant_id = b.id LEFT JOIN project c ON a.project_id = c.id LEFT JOIN users d ON a.creator_id = d.id LEFT JOIN cloud_vendor e ON a.vendor_id = e.id LEFT JOIN users f ON a.owner_id = f.id WHERE a.is_template = 0 AND a.is_deleted = 0 ");
        for (Param param : params) {
            Sign sign = param.getSign();
            if ("like".equals(sign.getSign())) {
                Map<String, Object> map = param.getParam();
                for (String key : map.keySet()) {
                    String value = String.valueOf(map.get(key));
                    switch (key) {
                        case "tenantName":
                            sql.append(" and b.name like '%").append(value).append("%'");
                            break;
                        case "projectName":
                            sql.append(" and c.name like '%").append(value).append("%'");
                            break;
                        case "targetName":
                            sql.append(" and d.name like '%").append(value).append("%' and a.catalog = 'User'");
                            break;
                        case "ownerName":
                            sql.append(" and f.name like '%").append(value).append("%'");
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        List<CloudServer> vms = Lists.newArrayList();
        sql = new StringBuilder(QueryBuilder.buildRaw(sql.toString(), new Pager(page, rows, null, sorter), ""));
        List<Object> list = this.list( CloudServer.class,sql.toString());
        for (Object object : list) {
            CloudServer cloudServer = JSONObject.parseObject(JSON.toJSONString(object), CloudServer.class);
            if (!"User".equals(cloudServer.getCatalog())) {
                cloudServer.setTargetName(null);
            }
            vms.add(cloudServer);
        }
        return vms;
    }

    public List<CloudServer> list() {
        String sql = "SELECT a.*,b.name AS host FROM cloud_server a LEFT JOIN server b ON a.host_id = b.id WHERE a.is_deleted = 0 AND a.is_template = 0 AND a.participators IS NOT NULL ";
        List<Object> list = this.list( CloudServer.class,sql);
        List<CloudServer> vms = Lists.newArrayList();
        for (Object object : list) {
            vms.add((CloudServer) object);
        }
        return vms;
    }
}