package com.bocloud.rms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.rms.entity.ServiceBill;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 账单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("serviceBillRepository")
public class ServiceBillRepository extends BasicGenericDao<ServiceBill, Long> {

    public ServiceBillRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceBill> list(int page, int rows, Map<String, String> sorter, List<Param> params,
                                  String serviceCode, String orderSn, List<Long> projectIds) {
        String sql =
                "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            sql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            sql += " and b.sn like '%" + orderSn + "%'";
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceBill.class, sql, paramMap);
    }

    public int count(List<Param> params, String serviceCode, String orderSn, List<Long> projectIds) {
        String sql =
                "select count(a.id) from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            sql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            sql += " and b.sn like '%" + orderSn + "%'";
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceBill> list(int page, int rows, Map<String, String> sorter, List<Param> params,
                                  String serviceCode, String orderSn, Long userId, List<Long> projectIds) {
        String subSql =
                "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            subSql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            subSql += " and b.sn like '%" + orderSn + "%'";
        }
        subSql = QueryBuilder.buildRaw(subSql, params, null, "a");
        String sql = "select distinct d.* from (" + subSql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "d");
        return this.list(ServiceBill.class, sql, paramMap);
    }

    public int count(List<Param> params, String serviceCode, String orderSn, Long userId, List<Long> projectIds) {
        String subSql =
                "select  a.* from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            subSql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            subSql += " and b.sn like '%" + orderSn + "%'";
        }
        subSql = QueryBuilder.buildRaw(subSql, params, null, "a");
        String sql = "select distinct count(1) from (" + subSql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, null, null, "d");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(String status, String chargeMode, Long tenantId, Long userId, Long orderId) {
        String sql =
                "select count(a.id) from service_bill a left join service_order b on b.id = a.order_id where b.charge_mode = :chargeMode and a.status = :status and a.order_id = :orderId ";
        Map<String, Object> paramMap = MapTools.simpleMap("chargeMode", chargeMode);
        paramMap.put("status", status);
        paramMap.put("orderId", orderId);
        if (null != tenantId && 0L != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            paramMap.put("tenantId", tenantId);
        }
        if (null != userId && 0L != userId) {
            sql += " and a.user_id = :userId";
            paramMap.put("userId", userId);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "delete from service_bill where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

}
