package com.bocloud.rms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.entity.ServiceOrder;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 订单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("serviceOrderRepository")
public class ServiceOrderRepository extends BasicGenericDao<ServiceOrder, Long> {

    public ServiceOrderRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long userId,
                                   List<Long> projectIds, String uuid) {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String subquerySql;
        if (StringUtils.hasText(uuid)) {
            subquerySql =
                    "select distinct a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                            + " left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                            + " left join cloud_service e on e.code = a.service_code "
                            + " left join project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            subquerySql =
                    "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                            + " left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                            + " left join cloud_service e on e.code = a.service_code "
                            + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        subquerySql = QueryBuilder.buildRaw(subquerySql, params, null, "a");
        String sql = "select distinct d.* from (" + subquerySql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "d");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public int count(List<Param> params, Long userId, List<Long> projectIds, String uuid) {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String subquerySql;
        if (StringUtils.hasText(uuid)) {
            subquerySql =
                    "select distinct a.* from service_order a left join order_item b on a.id = b.order_id where a.is_deleted = 0 and b.sn like '%" + uuid + "%' ";
        } else {
            subquerySql = "select a.* from service_order a where a.is_deleted = 0  ";
        }
        subquerySql = QueryBuilder.buildRaw(subquerySql, params, null, "a");
        String sql = "select distinct count(1) from (" + subquerySql + ") c  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or c.user_id =:userId";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or c.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += ")";
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql =
                "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                        + " left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                        + " left join cloud_service e on e.code = a.service_code "
                        + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params, String uuid) {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.hasText(uuid)) {
            sql =
                    "select distinct a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                            + " left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                            + " left join cloud_service e on e.code = a.service_code "
                            + " left join project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            sql =
                    "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                            + " left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                            + " left join cloud_service e on e.code = a.service_code "
                            + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        }

        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(id) from service_order a where a.is_deleted = 0  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(List<Param> params, String uuid) {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.hasText(uuid)) {
            sql =
                    "select distinct count(a.id) from service_order a left join order_item b on a.id = b.order_id where a.is_deleted = 0 and b.sn like '%" + uuid + "%'  ";
        } else {
            sql = "select count(a.id) from service_order a where a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "update service_order set is_deleted = 1, where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
//        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public ServiceOrder query(Long id) {
        String sql =
                "select a.*,b.name user_name,c.name tenant_name,p.name project_name from service_order a left join users b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join project p on p.id = a.project_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ServiceOrder> listByTenantId(String start, String end, Long tenantId) {
        String sql =
                "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                        + "left join cloud_service cs on cs.code=a.service_code left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }
}
