package com.bocloud.rms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.common.utils.DateUtils;
import com.bocloud.rms.common.utils.MathUtils;
import com.bocloud.rms.common.utils.StringUtil;
import com.bocloud.rms.entity.CloudService;
import com.bocloud.rms.entity.OrderItem;
import com.bocloud.rms.enums.BocloudStatus;
import com.bocloud.rms.enums.TimeUnit;
import com.bocloud.rms.internel.entity.*;
import com.bocloud.rms.intf.OperationReportService;
import com.bocloud.rms.repository.CloudServiceRepository;
import com.bocloud.rms.repository.OrderItemRepository;
import com.bocloud.rms.repository.ServiceOrderRepository;
import com.bocloud.rms.service.internal.CmsInternalService;
import com.bocloud.rms.service.internal.ImsInternalService;
import com.bocloud.rms.service.internal.ImsVolumePoolService;
import com.bocloud.rms.service.internal.SmsTenantService;
import com.bocloud.rms.service.utils.DateUtil;
import com.bocloud.rms.service.utils.ExcelChartUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.stream.Collectors.groupingBy;

/**
 * Created by liuyuanyuan on 2020/7/11 10:26
 */
@Service("operationReportService")
public class OperationReportServiceImpl implements OperationReportService {
    private final Logger logger = LoggerFactory.getLogger(OperationReportService.class);
    // sonar : format非线程安全方法，故更改作用域
//    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    @Autowired
    private ImsInternalService imsInternalService;
    @Autowired
    private SmsTenantService tenantService;
    @Autowired
    private ServiceOrderRepository serviceOrderRepository;
    @Autowired
    private CloudServiceRepository cloudServiceRepository;
    @Autowired
    private ImsVolumePoolService imsVolumePoolService;
    @Autowired
    private OrderItemRepository orderItemRepository;
    @Autowired
    private CmsInternalService cmsInternalService;

    @Override
    public BocloudResult resourceData(Long vendorId, Long vdcId, Long clusterId, String name, String category, Long tenantId, String vendorType, String time, String startTime, String endTime) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, String> timeMap = DateUtil.timeToMap(time);
        // 获取时间
        String unit = timeMap.get("unit");
        if (StringUtil.isEmpty(unit)) {
            unit = "WeekDays";
        } else {
            startTime = timeMap.get("start");
            endTime = timeMap.get("end");
        }
        switch (category) {
            case "computer":
                return getComputerResource(vendorId, vdcId, clusterId, startTime, endTime, unit, name, vendorType);
            case "topOrganization":
                return getTopOrganizationResource(vendorId, startTime, endTime, unit, name, tenantId);
            case "ip":
                return getIpResource(startTime, endTime, unit, name);
            case "tenant":
                return getTenantResource(vendorId, startTime, endTime, unit, tenantId);
            case "storage":
                return getStorageResource(vendorId, startTime, endTime, unit, name);
            default:
                break;
        }
        return new BocloudResult(false, "查询失败");
    }

    public List<CloudServer> listCloudServers(List<Long> tenantIds, Boolean isAll, List<Long> projectIds,
                                              String vendorType) {
        List<CloudServer> cloudServerList = imsInternalService.listVms(tenantIds, isAll, projectIds);
        // 云平台类型过滤
        if (!StringUtil.isEmpty(vendorType)) {
            cloudServerList = cloudServerList.stream()
                    .filter(cloudServer -> vendorType.equalsIgnoreCase(cloudServer.getVendorType()))
                    .collect(Collectors.toList());
        }
        return cloudServerList;
    }

    public List<Tenant> listTenants(Long tenantId) {
        try {
            // 获取指定条件下的主机列表
            Map<String, String> sorter = Maps.newHashMap();
            sorter.put("gmtCreate", Common.ONE);
            List<Param> params = new ArrayList<Param>();
            // 获取全部局办单位
            params.add(new Param(MapTools.simpleMap("type", "topOrganization"), Sign.EQ));
            if (null != tenantId) {
                params.add(new Param(MapTools.simpleMap("id", tenantId), Sign.EQ));
            }
            BocloudResult parentTenants = tenantService.listParentTenant(1, Integer.MAX_VALUE, params, sorter);

            JSONObject parentTenantsObject = JSONObject.parseObject(JSONObject.toJSONString(parentTenants.getData()));
            return parentTenantsObject.getJSONArray("rows").toJavaList(Tenant.class);
        } catch (Exception e) {
            logger.error("查询租户列表异常：" + e.getMessage());
            return null;
        }
    }

    private BocloudResult getTopOrganizationResource(Long vendorId, String start, String end, String unit, String name,
                                                     Long tenantId) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> dataResult = Maps.newHashMap();
        try {
            List<String> keys = new ArrayList<>();
            List<Map<String, Object>> values = new ArrayList<>();
            // 获取指定条件下的主机列表
            List<Tenant> tenants = listTenants(tenantId);
            List<Long> tenantIds = tenants.stream().map(Tenant::getId).collect(Collectors.toList());
            List<CloudServer> cloudServerList = listCloudServers(tenantIds, Boolean.FALSE, null, null);
            Date endTime = DateUtils.stringToDate(end, "yyyy-MM-dd HH:mm:ss");
            Date startTime = DateUtils.stringToDate(start, "yyyy-MM-dd HH:mm:ss");
            List<Integer> cpuData = new ArrayList<>();
            List<Double> memData = new ArrayList<>();
            List<Integer> vmCountData = new ArrayList<>();
            Map<String, Object> cpuMap = Maps.newHashMap();
            Map<String, Object> memMap = Maps.newHashMap();
            Map<String, Object> vmCountMap = Maps.newHashMap();
            while (startTime.before(timeUnitAdd(unit, endTime))) {
                keys.add(DATE_FORMAT.format(startTime));
                Date finalStartTime = startTime;
                // 筛选目标虚机 在 指定时间前创建 指定条件主机下 非创建中的虚机
                List<CloudServer> targetVms = cloudServerList.stream()
                        .filter(cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus())
                                && cloudServer.getGmtCreate().before(finalStartTime))
                        .collect(Collectors.toList());
                // cpu
                Integer cpuNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0)).reduce(Integer::sum)
                        .orElse(0);
                cpuData.add(cpuNum);
                ArrayList<Integer> cpuTimeData = new ArrayList<>(cpuData);
                cpuMap.put("name", "CPU(C)");
                cpuMap.put("data", cpuTimeData);
                // 内存
                BigDecimal memNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                memData.add(memNum.doubleValue());
                ArrayList<Double> memTimeData = new ArrayList<>(memData);
                memMap.put("name", "内存(GB)");
                memMap.put("data", memTimeData);
                // 虚机个数
                vmCountData.add(targetVms.size());
                ArrayList<Integer> vmCountTimeData = new ArrayList<>(vmCountData);
                vmCountMap.put("name", "虚机(个)");
                vmCountMap.put("data", vmCountTimeData);
                startTime = timeUnitAdd(unit, startTime);
            }
            values.add(cpuMap);
            values.add(memMap);
            values.add(vmCountMap);
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            dataResult.put("keys", timeKeys);
            dataResult.put("values", values);
            return new BocloudResult(true, dataResult, "查询成功");
        } catch (Exception e) {
            logger.error("get computer data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }

    }

    @Override
    public BocloudResult monitorData(Long vendorId, String name, String category, String type, Integer limit, String time) {
        limit = limit == null ? 5 : limit;
        Map<String, String> timeMap = DateUtil.timeToMap(time);
        switch (category) {
            case "dailyAlarm":
                return getDailyAlarm(vendorId, timeMap.get("start"), timeMap.get("end"), timeMap.get("unit"), name);
            case "Top5":
                return getTop5(vendorId, type, limit);
            default:
                break;
        }
        return new BocloudResult(false, "查询失败");
    }

    private BocloudResult getTop5(Long vendorId, String type, Integer limit) {
        try {
            return cmsInternalService.getTop5(vendorId, type, limit);
        } catch (Exception e) {
            logger.error("get storage data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    @Override
    public BocloudResult tenantData(int page, int rows, List<Param> params, Map<String, String> sorter, boolean simple,
                                    String condition, RequestContext context) {
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            Iterator<Param> iterator = params.iterator();
            Long vendorId = null;
            while (iterator.hasNext()) {
                Param param = iterator.next();
                if (param.getParam().containsKey("vendorId")) {
                    vendorId = Long.valueOf(param.getParam().get("vendorId").toString());
                    param.getParam().remove("vendorId");
                    break;
                }
            }
            BocloudResult ParentTenants = tenantService.listParentTenant(page, rows, params, sorter);
            JSONObject ParentTenantsObject = JSONObject.parseObject(JSONObject.toJSONString(ParentTenants.getData()));
            Long count = ParentTenantsObject.getLong("total");
            List<Tenant> tenants = ParentTenantsObject.getJSONArray("rows").toJavaList(Tenant.class);
            List<Long> orderIds = serviceOrderRepository.listByTenantId(null, null, null).stream()
                    .map(serviceOrder -> serviceOrder.getId()).collect(Collectors.toList());
            List<OrderItem> orderItems = orderItemRepository.queryTenantResource(orderIds);
            Map<Long, List<OrderItem>> ordersMap = orderItems.stream()
                    .filter(orderItem -> null != orderItem.getTenantId()).collect(groupingBy(OrderItem::getTenantId));
            List<Map<String, Object>> result = Lists.newArrayList();
            int cloudServerTotal = 0;
            int vpcTotal = 0;
            int volumeTotal = 0;
            int ipTotal = 0;
            int mysqlTotal = 0;
            int redisTotal = 0;
            int mongodbTotal = 0;
            int k8sTotal = 0;
            int postgresqlTotal = 0;
            int sqlserverTotal = 0;
            int rabbitmqTotal = 0;
            int kafkaTotal = 0;
            for (Tenant tenant : tenants) {
                List<Long> tenantIds;
                BocloudResult<List<Tenant>> children = tenantService.listChildren(tenant.getId());
                tenantIds = new ArrayList<>();
                if (children.getData() != null) {
                    tenantIds = children.getData().stream().map(Tenant::getId).collect(Collectors.toList());
                }
                tenantIds.add(tenant.getId());
                List<Long> finalTenantIds = tenantIds;
                Map<String, List<OrderItem>> tenantOrders;
                if (null != vendorId) {
                    Long finalVendorId = vendorId;
                    tenantOrders = ordersMap.values().stream().flatMap(Collection::stream)
                            .filter(orderItem -> finalTenantIds.contains(orderItem.getTenantId())
                                    && finalVendorId.equals(orderItem.getVendorId()))
                            .collect(groupingBy(OrderItem::getServiceCode));
                } else {
                    tenantOrders = ordersMap.values().stream().flatMap(Collection::stream)
                            .filter(orderItem -> finalTenantIds.contains(orderItem.getTenantId()))
                            .collect(groupingBy(OrderItem::getServiceCode));
                }
                int cloudServerCount = 0;
                int vpcCount = 0;
                int volumeCount = 0;
                int ipCount = 0;
                int mysqlCount = 0;
                int redisCount = 0;
                int mongodbCount = 0;
                int k8sCount = 0;
                int postgresqlCount = 0;
                int sqlserverCount = 0;
                int rabbitmqCount = 0;
                int kafkaCount = 0;
                for (String code : tenantOrders.keySet()) {
                    switch (code) {
                        // 云主机
                        case "huawei.standard.server":
                        case "tencent.standard.server":
                        case "tce.standard.server":
                        case "aliyun.standard.server":
                        case "manageone.standard.server":
                        case "azure.standard.server":
                        case "openstack.standard.server":
                        case "vmware.gpu.server":
                        case "vmware.standard.server":
                        case "smartx.standard.server":
                        case "fusionsphere.standard.server":
                        case "easystack.standard.server":
                            cloudServerCount += tenantOrders.get(code).size();
                            cloudServerTotal += tenantOrders.get(code).size();
                            break;
                        // 容器云
                        case "vmware.k8s.cluster":
                            k8sCount += tenantOrders.get(code).size();
                            k8sTotal += tenantOrders.get(code).size();
                            break;
                        // vpc
                        case "azure.standard.vpc":
                        case "huawei.standard.vpc":
                        case "tencent.standard.vpc":
                        case "aliyun.standard.vpc":
                        case "manageone.standard.vpc":
                        case "tce.standard.vpc":
                        case "openstack.standard.vpc":
                        case "vmware.nsx.vpc":
                        case "easystack.standard.vpc":
                            vpcCount += tenantOrders.get(code).size();
                            vpcTotal += tenantOrders.get(code).size();
                            break;
                        // 云硬盘
                        case "azure.standard.volume":
                        case "aliyun.standard.volume":
                        case "huawei.standard.volume":
                        case "tencent.standard.volume":
                        case "manageone.standard.volume":
                        case "tce.standard.cbs":
                        case "openstack.standard.volume":
                        case "smartx.standard.volume":
                        case "vmware.storage.disk":
                        case "fusionsphere.standard.volume":
                            volumeCount += tenantOrders.get(code).size();
                            volumeTotal += tenantOrders.get(code).size();
                            break;
                        // 弹性ip
                        case "vmware.nsx.eip":
                            ipCount += tenantOrders.get(code).size();
                            ipTotal += tenantOrders.get(code).size();
                            break;
                        case "rds.postgresql":
                            postgresqlCount += tenantOrders.get(code).size();
                            postgresqlTotal += tenantOrders.get(code).size();
                            break;
                        case "rds.sqlserver":
                            sqlserverCount += tenantOrders.get(code).size();
                            sqlserverTotal += tenantOrders.get(code).size();
                            break;
                        case "rds.mysql":
                            mysqlCount += tenantOrders.get(code).size();
                            mysqlTotal += tenantOrders.get(code).size();
                            break;
                        case "kvs.redis":
                            redisCount += tenantOrders.get(code).size();
                            redisTotal += tenantOrders.get(code).size();
                            break;
                        case "kvs.mongodb":
                            mongodbCount += tenantOrders.get(code).size();
                            mongodbTotal += tenantOrders.get(code).size();
                            break;
                        case "cms.rabbitmq":
                            rabbitmqCount += tenantOrders.get(code).size();
                            rabbitmqTotal += tenantOrders.get(code).size();
                            break;
                        case "cms.kafka":
                            kafkaCount += tenantOrders.get(code).size();
                            kafkaTotal += tenantOrders.get(code).size();
                            break;
                        default:
                            break;
                    }
                }
                Map<String, Object> tenantResourceCount = Maps.newHashMap();
                tenantResourceCount.put("name", tenant.getName());
                tenantResourceCount.put("cloudServerCount", cloudServerCount);
                tenantResourceCount.put("k8sCount", k8sCount);
                tenantResourceCount.put("vpcCount", vpcCount);
                tenantResourceCount.put("volumeCount", volumeCount);
                tenantResourceCount.put("ipCount", ipCount);
                tenantResourceCount.put("postgresqlCount", postgresqlCount);
                tenantResourceCount.put("sqlserverCount", sqlserverCount);
                tenantResourceCount.put("mysqlCount", mysqlCount);
                tenantResourceCount.put("redisCount", redisCount);
                tenantResourceCount.put("mongodbCount", mongodbCount);
                tenantResourceCount.put("rabbitmqCount", rabbitmqCount);
                tenantResourceCount.put("kafkaCount", kafkaCount);
                Stream<Integer> stream = Arrays.stream(
                        new Integer[]{cloudServerCount, vpcCount, volumeCount, ipCount, mysqlCount, redisCount,
                                mongodbCount, k8sCount, postgresqlCount, sqlserverCount, rabbitmqCount, kafkaCount});
                Integer reduce = stream.reduce(0, Integer::sum);
                tenantResourceCount.put("subtotal", reduce);
                result.add(tenantResourceCount);
            }
            Map<String, Object> tenantResourceTotal = Maps.newHashMap();
            tenantResourceTotal.put("name", "合计");
            tenantResourceTotal.put("cloudServerCount", cloudServerTotal);
            tenantResourceTotal.put("k8sCount", k8sTotal);
            tenantResourceTotal.put("vpcCount", vpcTotal);
            tenantResourceTotal.put("volumeCount", volumeTotal);
            tenantResourceTotal.put("ipCount", ipTotal);
            tenantResourceTotal.put("postgresqlCount", postgresqlTotal);
            tenantResourceTotal.put("sqlserverCount", sqlserverTotal);
            tenantResourceTotal.put("mysqlCount", mysqlTotal);
            tenantResourceTotal.put("redisCount", redisTotal);
            tenantResourceTotal.put("mongodbCount", mongodbTotal);
            tenantResourceTotal.put("rabbitmqCount", rabbitmqTotal);
            tenantResourceTotal.put("kafkaCount", kafkaTotal);
            Stream<Integer> stream = Arrays
                    .stream(new Integer[]{cloudServerTotal, vpcTotal, volumeTotal, ipTotal, mysqlTotal, redisTotal,
                            mongodbTotal, k8sTotal, postgresqlTotal, sqlserverTotal, rabbitmqTotal, kafkaTotal});
            tenantResourceTotal.put("subtotal", stream.reduce(0, Integer::sum));
            result.add(tenantResourceTotal);
            GridBean gridBean = GridHelper.getBean(page, rows, count.intValue(), result);
            // 获取列资源
            List<Param> paramsData = Lists.newArrayList();
            paramsData.add(new Param(MapTools.simpleMap("status", "PUBLISHED"), Sign.EQ));
            List<CloudService> cloudServices = cloudServiceRepository.list(1, Integer.MAX_VALUE, null, paramsData,
                    null);
            Map<String, CloudService> cloudServiceMap = Maps.newHashMap();
            for (CloudService cloudService : cloudServices) {
                cloudServiceMap.put(cloudService.getCode(), cloudService);
            }
            List<Map<String, Object>> columns = Lists.newArrayList();
            Boolean cloudServerIsRelease = false;
            Boolean vpcIsRelease = false;
            Boolean volumeIsRelease = false;
            for (String code : cloudServiceMap.keySet()) {
                Map<String, Object> column;
                if ("PUBLISHED".equals(cloudServiceMap.get(code).getStatus())) {
                    switch (code) {
                        // 云主机
                        case "huawei.standard.server":
                        case "tencent.standard.server":
                        case "tce.standard.server":
                        case "aliyun.standard.server":
                        case "manageone.standard.server":
                        case "azure.standard.server":
                        case "openstack.standard.server":
                        case "vmware.gpu.server":
                        case "vmware.standard.server":
                        case "smartx.standard.server":
                        case "fusionsphere.standard.server":
                        case "easystack.standard.server":
                            if (!cloudServerIsRelease) {
                                column = Maps.newHashMap();
                                column.put("name", "云主机");
                                column.put("code", "cloudServerCount");
                                columns.add(column);
                                cloudServerIsRelease = true;
                            }
                            break;
                        // 容器云
                        case "vmware.k8s.cluster":
                            column = Maps.newHashMap();
                            column.put("name", "容器云");
                            column.put("code", "k8sCount");
                            columns.add(column);
                            break;
                        // vpc
                        case "azure.standard.vpc":
                        case "huawei.standard.vpc":
                        case "tencent.standard.vpc":
                        case "aliyun.standard.vpc":
                        case "manageone.standard.vpc":
                        case "tce.standard.vpc":
                        case "openstack.standard.vpc":
                        case "vmware.nsx.vpc":
                        case "easystack.standard.vpc":
                            if (!vpcIsRelease) {
                                column = Maps.newHashMap();
                                column.put("name", "VPC");
                                column.put("code", "vpcCount");
                                columns.add(column);
                                vpcIsRelease = true;
                            }
                            break;
                        // 云硬盘
                        case "azure.standard.volume":
                        case "aliyun.standard.volume":
                        case "huawei.standard.volume":
                        case "tencent.standard.volume":
                        case "manageone.standard.volume":
                        case "tce.standard.cbs":
                        case "openstack.standard.volume":
                        case "smartx.standard.volume":
                        case "vmware.storage.disk":
                        case "fusionsphere.standard.volume":
                        case "easystack.standard.volume":
                            if (!volumeIsRelease) {
                                column = Maps.newHashMap();
                                column.put("name", "云硬盘");
                                column.put("code", "volumeCount");
                                columns.add(column);
                                volumeIsRelease = true;
                            }
                            break;
                        // 弹性ip
                        case "vmware.nsx.eip":
                            column = Maps.newHashMap();
                            column.put("name", "弹性IP");
                            column.put("code", "ipCount");
                            columns.add(column);
                            break;
                        case "rds.postgresql":
                            column = Maps.newHashMap();
                            column.put("name", "PostgreSQL");
                            column.put("code", "postgresqlCount");
                            columns.add(column);
                            break;
                        case "rds.sqlserver":
                            column = Maps.newHashMap();
                            column.put("name", "SQLServer");
                            column.put("code", "sqlserverCount");
                            columns.add(column);
                            break;
                        case "rds.mysql":
                            column = Maps.newHashMap();
                            column.put("name", "MySQL");
                            column.put("code", "mysqlCount");
                            columns.add(column);
                            break;
                        case "kvs.redis":
                            column = Maps.newHashMap();
                            column.put("name", "Redis");
                            column.put("code", "redisCount");
                            columns.add(column);
                            break;
                        case "kvs.mongodb":
                            column = Maps.newHashMap();
                            column.put("name", "MongoDB");
                            column.put("code", "mongodbCount");
                            columns.add(column);
                            break;
                        case "cms.rabbitmq":
                            column = Maps.newHashMap();
                            column.put("name", "RabbitMQ");
                            column.put("code", "rabbitmqCount");
                            columns.add(column);
                            break;
                        case "cms.kafka":
                            column = Maps.newHashMap();
                            column.put("name", "Kafka");
                            column.put("code", "kafkaCount");
                            columns.add(column);
                            break;
                        default:
                            break;
                    }
                }
            }
            Map<String, Object> data = Maps.newHashMap();
            data.put("page", gridBean.getPage());
            data.put("rows", gridBean.getRows());
            data.put("total", gridBean.getTotal());
            data.put("pages", gridBean.getPages());
            data.put("columns", JSONObject.toJSON(columns));
            return new BocloudResult(true, data, "查询成功");
        } catch (Exception e) {
            logger.error("Query tenant resource count fail:", e);
            return new BocloudResult(false, "查询失败", null, null);
        }
    }

    private BocloudResult getDailyAlarm(Long vendorId, String start, String end, String unit, String StorageName) {
        try {
            return cmsInternalService.getDailyAlarm(start, end);
        } catch (Exception e) {
            logger.error("get storage data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    @Override
    public BocloudResult computerData(Long vendorId, Long vdcId, Long clusterId, String name, String vendorType, JSONObject paging) {
        List<JSONObject> tables = new ArrayList<>();
        BocloudResult hostResult = imsInternalService.getHostList(vendorId, vdcId, clusterId, name, vendorType);

        Assert.notNull(hostResult.getData(), "主机信息获取失败");
        List<Server> servers = JSONArray.parseArray(JSONObject.toJSONString(hostResult.getData()), Server.class);
        if (null != vendorId) {
            servers = servers.stream().filter(server -> vendorId.equals(server.getVendorId())).collect(Collectors.toList());
        }
        List<Long> serverIds = servers.stream().map(server -> server.getId()).collect(Collectors.toList());
        // 计算虚机数量
        if (!serverIds.isEmpty()) {
            BocloudResult vmsResult = imsInternalService.getVmList(serverIds);
            Assert.notNull(hostResult.getData(), "虚机信息获取失败");
            List<CloudServer> cloudServerList = JSONArray.parseArray(JSONObject.toJSONString(vmsResult.getData()),
                    CloudServer.class);
            List<CloudServer> cloudServers = new ArrayList<>();
            // 如果是模板，则不统计在内
            for (CloudServer cloudServer : cloudServerList) {
                if (null == cloudServer.getIsTemplate() || !cloudServer.getIsTemplate()) {
                    cloudServers.add(cloudServer);
                }
            }
            Map<Long, List<CloudServer>> vmMap = cloudServers.stream()
                    .filter(cloudServer -> serverIds.contains(cloudServer.getHostId()))
                    .collect(groupingBy(CloudServer::getHostId));
            for (Server server : servers) {
                ServerConfig config = server.getConfig();
                if (null == config) {
                    continue;
                }
                JSONObject serverObj = new JSONObject();
                serverObj.put("id", server.getId());
                serverObj.put("name", server.getName());
                serverObj.put("vendorName", server.getVendorName());
                serverObj.put("vdcName", server.getVdcName());
                serverObj.put("clusterName", server.getClusterName());
                serverObj.put("vmNum", null == vmMap.get(server.getId()) ? 0 : vmMap.get(server.getId()).size());
                serverObj.put("vcpu", config.getCpuHz());
                serverObj.put("vcpuUsed", config.getCpuUsed());
                if (null != config.getCpuHz()) {
                    serverObj.put("vcpuPercent",
                            new BigDecimal(null == config.getCpuUsed() ? 0 : config.getCpuUsed())
                                    .divide(BigDecimal.valueOf(config.getCpuHz()), 5, RoundingMode.HALF_UP)
                                    .multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                } else {
                    serverObj.put("vcpuPercent", 0);
                }
                serverObj.put("memory", config.getMemoryCapacity());
                serverObj.put("memoryUsed", config.getMemUsed());
                if (null != config.getMemoryCapacity()) {
                    serverObj.put("memoryPercent",
                            BigDecimal.valueOf(null == config.getMemUsed() ? 0 : config.getMemUsed())
                                    .divide(new BigDecimal(config.getMemoryCapacity()), 5, RoundingMode.HALF_UP)
                                    .multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                } else {
                    serverObj.put("memoryPercent", 0);
                }
                tables.add(serverObj);
            }
            // 数据分页
            if (null != paging) {
                int pageSize = paging.getLong("rows").intValue();
                int pageNum = paging.getLong("page").intValue();
                if (ListTool.isEmpty(tables)) {
                    return new BocloudResult(true, tables, "查询成功");
                }
                Integer count = tables.size(); // 记录总数
                Integer pageCount = 0; // 页数
                if (count % pageSize == 0) {
                    pageCount = count / pageSize;
                } else {
                    pageCount = count / pageSize + 1;
                }
                int fromIndex = 0; // 开始索引
                int toIndex = 0; // 结束索引
                if (pageNum != pageCount) {
                    fromIndex = (pageNum - 1) * pageSize;
                    toIndex = fromIndex + pageSize;
                } else {
                    fromIndex = (pageNum - 1) * pageSize;
                    toIndex = count;
                }
                GridBean gridBean = GridHelper.getBean(pageNum, pageSize, count, tables.subList(fromIndex, toIndex));
                return new BocloudResult(true, gridBean, "查询成功");
            }else {
                // sonar : 上方已经if子句暗示paging可能为null，下发依旧调用了paging.getLong("page").intValue() 故返回null，上层对空指针做处理
                return new BocloudResult(false,null,"查询失败");
            }
        }
        GridBean gridBean = GridHelper.getBean(paging.getLong("page").intValue(), paging.getLong("rows").intValue(), 0,
                tables);
        return new BocloudResult(true, gridBean, "查询成功");
    }

    @Override
    public BocloudResult download(Long vendorId, Long vdcId, Long clusterId, Long id, String name, String category, String vendorType, String time) {
        SXSSFWorkbook workbook =null;
        try {
            // 获取数据
            JSONArray surface = null;
            BocloudResult result = resourceData(vendorId, null, clusterId, null, category, null, vendorType, time, null, null);
            JSONObject chartData = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
            JSONArray columns = null;
            String fileName = null;
            JSONObject computerChartData = null;
            switch (category) {
                case "computer":
                    JSONObject paging = new JSONObject();
                    paging.put("page", 1);
                    paging.put("rows", Integer.MAX_VALUE);
                    BocloudResult bocloudResult = computerData(vendorId, vdcId, clusterId, name, null, paging);
                    JSONObject object = JSONObject.parseObject(JSONObject.toJSONString(bocloudResult.getData()));
                    surface = object.getJSONArray("rows");
                    fileName = "计算资源报表";
                    break;
                case "ip":
                    BocloudResult poolList = imsInternalService.getIpPoolList(name);
                    JSONObject ipObject = JSONObject.parseObject(JSONObject.toJSONString(poolList.getData()));
                    surface = ipObject.getJSONArray("rows");
                    fileName = "IP资源报表";
                    break;
                case "tenant":
                    List<Param> tenantData = Lists.newArrayList();
                    if (null != vendorId) {
                        Map<String, Object> vendor = MapTools.simpleMap("vendorId", vendorId);
                        Map<String, Object> tenant = MapTools.simpleMap("id", id);
                        Param param = new Param(vendor, Sign.EQ);
                        param.setParam(tenant);
                        tenantData.add(param);
                    }
                    BocloudResult tenantResult = tenantData(1, Integer.MAX_VALUE, tenantData, null, false, null, null);
                    JSONObject tenantObject = JSONObject.parseObject(JSONObject.toJSONString(tenantResult.getData()));
                    surface = tenantObject.getJSONArray("rows");
                    columns = tenantObject.getJSONArray("columns");
                    fileName = "租户报表";
                    break;
                case "storage":
                    // 获取指定条件下存储的数量
                    List<Param> storageData = Lists.newArrayList();
                    if (null != vendorId) {
                        storageData.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
                    }
                    if (null != name) {
                        storageData.add(new Param(MapTools.simpleMap("name", storageData), Sign.LK));
                    }
                    BocloudResult storagePools = imsVolumePoolService.listByStoragePools(1, Integer.MAX_VALUE, storageData,
                            null);
                    JSONObject storageObject = JSONObject.parseObject(JSONObject.toJSONString(storagePools.getData()));
                    surface = storageObject.getJSONArray("rows");
                    fileName = "存储资源报表";
                    break;
                default:
                    break;
            }
            workbook = ExcelChartUtil.createExcel(category, chartData, surface, columns);
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("下载失败：", e);
            return new BocloudResult(false, "下载失败");
        } finally {
            if (null != workbook){
                try {
                    workbook.close();
                } catch (IOException e) {
                    logger.info("数据流关闭异常!");
                }
            }
        }
    }

    private BocloudResult getTenantResource(Long vendorId, String start, String end, String unit, Long tenantId) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> dataResult = Maps.newHashMap();
        try {
            List<String> keys = new ArrayList<>();
            List<Map<String, Object>> values = new ArrayList<>();
            // 获取指定条件下租户订购资源的数量
            List<Param> paramsData = Lists.newArrayList();
            if (null != tenantId) {
                paramsData.add(new Param(MapTools.simpleMap("id", tenantId), Sign.EQ));
            }
            BocloudResult ParentTenants = tenantService.listParentTenant(1, Integer.MAX_VALUE, paramsData, null);
            JSONObject ParentTenantsObject = JSONObject.parseObject(JSONObject.toJSONString(ParentTenants.getData()));
            List<Tenant> tenants = ParentTenantsObject.getJSONArray("rows").toJavaList(Tenant.class);
            List<Long> tenantIds = new ArrayList<>();
            for (Tenant tenant : tenants) {
                BocloudResult<List<Tenant>> children = tenantService.listChildren(tenant.getId());
                if (children.getData() != null) {
                    List<Long> ids = children.getData().stream().map(Tenant::getId).collect(Collectors.toList());
                    tenantIds.addAll(ids);
                }
                tenantIds.add(tenant.getId());
            }
            tenantIds.add(tenantId);
            // 云主机
            List<String> cloudServers = Arrays.asList("huawei.standard.server", "tencent.standard.server",
                    "tce.standard.server", "aliyun.standard.server", "manageone.standard.server",
                    "azure.standard.server", "openstack.standard.server", "vmware.gpu.server",
                    "vmware.standard.server", "smartx.standard.server", "fusionsphere.standard.server", "easystack.standard.server"
            );
            List<OrderItem> cloudServerServiceOrder = orderItemRepository.resourceCount(tenantIds, cloudServers,
                    vendorId);
            // vpc
            List<String> vpcs = Arrays.asList("azure.standard.vpc", "huawei.standard.vpc", "tencent.standard.vpc",
                    "aliyun.standard.vpc", "manageone.standard.vpc", "tce.standard.vpc", "openstack.standard.vpc",
                    "vmware.nsx.vpc", "easystack.standard.vpc");
            List<OrderItem> vpcServiceOrder = orderItemRepository.resourceCount(tenantIds, vpcs, vendorId);
            // 硬盘
            List<String> volumeServers = Arrays.asList("azure.standard.volume", "aliyun.standard.volume",
                    "huawei.standard.volume", "tencent.standard.volume", "manageone.standard.volume",
                    "tce.standard.cbs", "openstack.standard.volume", "smartx.standard.volume", "vmware.storage.disk",
                    "fusionsphere.standard.volume", "easystack.standard.volume"
            );
            List<OrderItem> volumeServiceOrder = orderItemRepository.resourceCount(tenantIds, volumeServers, vendorId);
            // 剩余订购资源
            List<OrderItem> orderItems = orderItemRepository.resourceCount(tenantIds, null, vendorId);
            Map<String, List<OrderItem>> collect = orderItems.stream().collect(groupingBy(OrderItem::getServiceCode));
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("status", "PUBLISHED"), Sign.EQ));
            List<CloudService> cloudServices = cloudServiceRepository.list(1, Integer.MAX_VALUE, null, paramsData,
                    null);
            Map<String, CloudService> cloudServiceMap = Maps.newHashMap();
            for (CloudService cloudService : cloudServices) {
                cloudServiceMap.put(cloudService.getCode(), cloudService);
            }
            Date endTime = DateUtils.stringToDate(end, "yyyy-MM-dd HH:mm:ss");
            Date startTime = DateUtils.stringToDate(start, "yyyy-MM-dd HH:mm:ss");
            List<Integer> csData = new ArrayList<>();
            List<Integer> vpcData = new ArrayList<>();
            List<Integer> k8sData = new ArrayList<>();
            List<Integer> volumeData = new ArrayList<>();
            List<Integer> sqlServerData = new ArrayList<>();
            List<Integer> kafkaData = new ArrayList<>();
            List<Integer> mysqlData = new ArrayList<>();
            List<Integer> redisData = new ArrayList<>();
            List<Integer> rabbitmqData = new ArrayList<>();
            List<Integer> mongodbData = new ArrayList<>();
            List<Integer> pgData = new ArrayList<>();
            List<Integer> ipData = new ArrayList<>();
            List<Integer> totalData = new ArrayList<>();
            Map<String, Object> csMap = Maps.newHashMap();
            Map<String, Object> vpcMap = Maps.newHashMap();
            Map<String, Object> k8sMap = Maps.newHashMap();
            Map<String, Object> volumeMap = Maps.newHashMap();
            Map<String, Object> sqlServerMap = Maps.newHashMap();
            Map<String, Object> kafkaMap = Maps.newHashMap();
            Map<String, Object> mysqlMap = Maps.newHashMap();
            Map<String, Object> redisMap = Maps.newHashMap();
            Map<String, Object> rabbitmqMap = Maps.newHashMap();
            Map<String, Object> mongodbMap = Maps.newHashMap();
            Map<String, Object> pgMap = Maps.newHashMap();
            Map<String, Object> ipMap = Maps.newHashMap();
            Map<String, Object> totalMap = Maps.newHashMap();
            while (startTime.before(timeUnitAdd(unit, endTime))) {
                keys.add(DATE_FORMAT.format(startTime));
                Date finalStartTime = startTime;
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                // 筛选订购资源 在指定时间前创建
                List<OrderItem> targetvms = cloudServerServiceOrder.stream()
                        .filter(orderItem -> orderItem.getEffectiveTime().before(finalStartTime) || formatter
                                .format(orderItem.getEffectiveTime()).equals(formatter.format(finalStartTime)))
                        .collect(Collectors.toList());
                List<OrderItem> targetVpcs = vpcServiceOrder.stream()
                        .filter(orderItem -> orderItem.getEffectiveTime().before(finalStartTime) || formatter
                                .format(orderItem.getEffectiveTime()).equals(formatter.format(finalStartTime)))
                        .collect(Collectors.toList());
                List<OrderItem> targetk8ses = getTarget(collect, finalStartTime, formatter, "vmware.k8s.cluster");
                List<OrderItem> targetVolumes = volumeServiceOrder.stream()
                        .filter(orderItem -> orderItem.getEffectiveTime().before(finalStartTime) || formatter
                                .format(orderItem.getEffectiveTime()).equals(formatter.format(finalStartTime)))
                        .collect(Collectors.toList());
                List<OrderItem> targetSqlServers = getTarget(collect, finalStartTime, formatter, "rds.sqlserver");
                List<OrderItem> targetKafkas = getTarget(collect, finalStartTime, formatter, "cms.kafka");
                List<OrderItem> targetMysqls = getTarget(collect, finalStartTime, formatter, "rds.mysql");
                List<OrderItem> targetRediss = getTarget(collect, finalStartTime, formatter, "kvs.redis");
                List<OrderItem> targetRabbitmqs = getTarget(collect, finalStartTime, formatter, "cms.rabbitmq");
                List<OrderItem> targetMongodbs = getTarget(collect, finalStartTime, formatter, "kvs.mongodb");
                List<OrderItem> targetPgs = getTarget(collect, finalStartTime, formatter, "rds.postgresql");
                List<OrderItem> targetIps = getTarget(collect, finalStartTime, formatter, "vmware.nsx.eip");

                // 云主机
                Integer csNum = targetvms.size();
                csData.add(csNum);
                ArrayList<Integer> csTimeData = new ArrayList<>(csData);
                csMap.put("name", "云主机(个)");
                csMap.put("data", csTimeData);
                // 容器云
                Integer k8sNum = targetk8ses.size();
                k8sData.add(k8sNum);
                ArrayList<Integer> k8sTimeData = new ArrayList<>(k8sData);
                k8sMap.put("name", "容器云(个)");
                k8sMap.put("data", k8sTimeData);
                // vpc
                Integer vpcNum = targetVpcs.size();
                vpcData.add(vpcNum);
                ArrayList<Integer> vpcTimeData = new ArrayList<>(vpcData);
                vpcMap.put("name", "VPC(个)");
                vpcMap.put("data", vpcTimeData);
                // 云硬盘
                Integer volumeNum = targetVolumes.size();
                volumeData.add(volumeNum);
                ArrayList<Integer> volumeTimeData = new ArrayList<>(volumeData);
                volumeMap.put("name", "云硬盘(个)");
                volumeMap.put("data", volumeTimeData);
                // sqlserver
                Integer sqlserverNum = targetSqlServers.size();
                sqlServerData.add(sqlserverNum);
                ArrayList<Integer> sqlserverTimeData = new ArrayList<>(sqlServerData);
                sqlServerMap.put("name", "SQLServer(个)");
                sqlServerMap.put("data", sqlserverTimeData);
                // kafka
                Integer kafkaNum = targetKafkas.size();
                kafkaData.add(kafkaNum);
                ArrayList<Integer> kafkaTimeData = new ArrayList<>(kafkaData);
                kafkaMap.put("name", "Kafka(个)");
                kafkaMap.put("data", kafkaTimeData);
                // mysql
                Integer mysqlNum = targetMysqls.size();
                mysqlData.add(mysqlNum);
                ArrayList<Integer> mysqlTimeData = new ArrayList<>(mysqlData);
                mysqlMap.put("name", "MySQL(个)");
                mysqlMap.put("data", mysqlTimeData);
                // redis
                Integer redisNum = targetRediss.size();
                redisData.add(redisNum);
                ArrayList<Integer> redisTimeData = new ArrayList<>(redisData);
                redisMap.put("name", "Redis(个)");
                redisMap.put("data", redisTimeData);
                // rabbitmq
                Integer rabbitmqNum = targetRabbitmqs.size();
                rabbitmqData.add(rabbitmqNum);
                ArrayList<Integer> rabbitmqTimeData = new ArrayList<>(rabbitmqData);
                rabbitmqMap.put("name", "RabbitMQ(个)");
                rabbitmqMap.put("data", rabbitmqTimeData);
                // mongodb
                Integer mongodbNum = targetMongodbs.size();
                mongodbData.add(mongodbNum);
                ArrayList<Integer> mongodbTimeData = new ArrayList<>(mongodbData);
                mongodbMap.put("name", "MondoDB(个)");
                mongodbMap.put("data", mongodbTimeData);
                // PG
                Integer pgNum = targetPgs.size();
                pgData.add(pgNum);
                ArrayList<Integer> pgTimeData = new ArrayList<>(pgData);
                pgMap.put("name", "PostgreSQL(个)");
                pgMap.put("data", pgTimeData);
                // 弹性ip
                Integer ipNum = targetIps.size();
                ipData.add(ipNum);
                ArrayList<Integer> ipTimeData = new ArrayList<>(ipData);
                ipMap.put("name", "弹性IP(个)");
                ipMap.put("data", ipTimeData);
                // 合计
                Stream<Integer> stream = Arrays.stream(
                        new Integer[]{targetvms.size(), targetk8ses.size(), targetVpcs.size(), targetVolumes.size(),
                                targetSqlServers.size(), targetKafkas.size(), targetMysqls.size(), targetRediss.size(),
                                targetRabbitmqs.size(), targetMongodbs.size(), targetPgs.size(), targetIps.size()});
                Integer total = stream.reduce(0, Integer::sum);
                totalData.add(total);
                ArrayList<Integer> totalTimeData = new ArrayList<>(totalData);
                totalMap.put("name", "合计(个)");
                totalMap.put("data", totalTimeData);
                startTime = timeUnitAdd(unit, startTime);
            }
            Boolean cloudServerIsRelease = false;
            Boolean vpcIsRelease = false;
            Boolean volumeIsRelease = false;
            for (String code : cloudServiceMap.keySet()) {
                if ("PUBLISHED".equals(cloudServiceMap.get(code).getStatus())) {
                    switch (code) {
                        // 云主机
                        case "huawei.standard.server":
                        case "tencent.standard.server":
                        case "tce.standard.server":
                        case "aliyun.standard.server":
                        case "manageone.standard.server":
                        case "azure.standard.server":
                        case "openstack.standard.server":
                        case "vmware.gpu.server":
                        case "vmware.standard.server":
                        case "smartx.standard.server":
                        case "fusionsphere.standard.server":
                        case "easystack.standard.server":
                            if (!cloudServerIsRelease) {
                                values.add(csMap);
                                cloudServerIsRelease = true;
                            }
                            break;
                        // 容器云
                        case "vmware.k8s.cluster":
                            values.add(k8sMap);
                            break;
                        // vpc
                        case "azure.standard.vpc":
                        case "huawei.standard.vpc":
                        case "tencent.standard.vpc":
                        case "aliyun.standard.vpc":
                        case "manageone.standard.vpc":
                        case "tce.standard.vpc":
                        case "openstack.standard.vpc":
                        case "vmware.nsx.vpc":
                        case "easystack.standard.vpc":
                            if (!vpcIsRelease) {
                                values.add(vpcMap);
                                vpcIsRelease = true;
                            }
                            break;
                        // 云硬盘
                        case "azure.standard.volume":
                        case "aliyun.standard.volume":
                        case "huawei.standard.volume":
                        case "tencent.standard.volume":
                        case "manageone.standard.volume":
                        case "tce.standard.cbs":
                        case "openstack.standard.volume":
                        case "smartx.standard.volume":
                        case "vmware.storage.disk":
                        case "fusionsphere.standard.volume":
                        case "easystack.standard.volume":
                            if (!volumeIsRelease) {
                                values.add(volumeMap);
                                volumeIsRelease = true;
                            }
                            break;
                        // 弹性ip
                        case "vmware.nsx.eip":
                            values.add(ipMap);
                            break;
                        case "rds.postgresql":
                            values.add(pgMap);
                            break;
                        case "rds.sqlserver":
                            values.add(sqlServerMap);
                            break;
                        case "rds.mysql":
                            values.add(mysqlMap);
                            break;
                        case "kvs.redis":
                            values.add(redisMap);
                            break;
                        case "kvs.mongodb":
                            values.add(mongodbMap);
                            break;
                        case "cms.rabbitmq":
                            values.add(rabbitmqMap);
                            break;
                        case "cms.kafka":
                            values.add(kafkaMap);
                            break;
                        default:
                            break;
                    }
                }
            }
            values.add(totalMap);
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            dataResult.put("keys", timeKeys);
            dataResult.put("values", values);
            return new BocloudResult(true, dataResult, "查询成功");
        } catch (Exception e) {
            logger.error("get tenant resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    private List<OrderItem> getTarget(Map<String, List<OrderItem>> collect, Date finalStartTime, SimpleDateFormat formatter, String s) {
        return null == collect.get(s) ? Lists.newArrayList()
                : collect.get(s).stream()
                .filter(orderItem -> orderItem.getEffectiveTime().before(finalStartTime) || formatter
                        .format(orderItem.getEffectiveTime()).equals(formatter.format(finalStartTime)))
                .collect(Collectors.toList());
    }

    private BocloudResult getStorageResource(Long vendorId, String start, String end, String unit, String StorageName) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> dataResult = Maps.newHashMap();
        try {
            List<String> keys = new ArrayList<>();
            List<Map<String, Object>> values = new ArrayList<>();
            // 获取指定条件下存储的数量
            List<Param> paramsData = Lists.newArrayList();
            if (null != vendorId) {
                paramsData.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
            }
            if (null != StorageName) {
                paramsData.add(new Param(MapTools.simpleMap("name", StorageName), Sign.LK));
            }
            BocloudResult result = imsVolumePoolService.listByStoragePools(1, Integer.MAX_VALUE, paramsData, null);
            JSONObject storageObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
            List<VolumePool> volumePools = storageObject.getJSONArray("rows").toJavaList(VolumePool.class);
            Date endTime = DateUtils.stringToDate(end, "yyyy-MM-dd HH:mm:ss");
            Date startTime = DateUtils.stringToDate(start, "yyyy-MM-dd HH:mm:ss");
            List<Double> storageData = new ArrayList<>();
            Map<String, Object> storageMap = Maps.newHashMap();
            while (startTime.before(timeUnitAdd(unit, endTime))) {
                keys.add(DATE_FORMAT.format(startTime));
                Date finalStartTime = startTime;
                // 筛选存储 在指定时间前创建
                List<VolumePool> targetStorages = volumePools.stream()
                        .filter(volumePool -> volumePool.getGmtCreate().before(finalStartTime))
                        .collect(Collectors.toList());
                Double capacityNum = targetStorages.stream()
                        .map(volumePool -> Optional.ofNullable(volumePool.getCapacity()).orElse(0.0))
                        .reduce(Double::sum).orElse(0.0);
                // 容量
                storageData.add(capacityNum);
                ArrayList<Double> storageTimeData = new ArrayList<>(storageData);
                storageMap.put("name", "存储(GB)");
                storageMap.put("data", storageTimeData);
                startTime = timeUnitAdd(unit, startTime);
            }
            values.add(storageMap);
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            dataResult.put("keys", timeKeys);
            dataResult.put("values", values);
            return new BocloudResult(true, dataResult, "查询成功");
        } catch (Exception e) {
            logger.error("get storage data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    private BocloudResult getIpResource(String start, String end, String unit, String poolName) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> dataResult = Maps.newHashMap();
        try {
            List<String> keys = new ArrayList<>();
            List<Map<String, Object>> values = new ArrayList<>();
            // 获取指定条件下ip池中ip的数量
            BocloudResult result = imsInternalService.getIpPoolList(poolName);
            if (result.isFailed()) {
                return result;
            }
            JSONObject ipPoolObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
            List<IpPool> ipPools = ipPoolObject.getJSONArray("rows").toJavaList(IpPool.class);
            if (null == ipPools) {
                ipPools = new ArrayList<>();
            }
            BigInteger ipTotal = new BigInteger("0");
            for (IpPool ipPool : ipPools) {
                ipTotal = ipTotal.add(ipPool.getIpTotal());
            }
            // 获取指定条件下的ip列表
            BocloudResult ipResult = imsInternalService.getIpNumber(poolName);
            if (ipResult.isFailed()) {
                return ipResult;
            }
            JSONObject ipUsedObject = JSONObject.parseObject(JSONObject.toJSONString(ipResult.getData()));
            List<Ip> ips = ipUsedObject.getJSONArray("rows").toJavaList(Ip.class);
            if (null == ips) {
                ips = new ArrayList<>();
            }
            Date endTime = DateUtils.stringToDate(end, "yyyy-MM-dd HH:mm:ss");
            Date startTime = DateUtils.stringToDate(start, "yyyy-MM-dd HH:mm:ss");
            List<Integer> usedData = new ArrayList<>();
            List<Double> percentData = new ArrayList<>();
            Map<String, Object> usedMap = Maps.newHashMap();
            Map<String, Object> percentMap = Maps.newHashMap();
            while (startTime.before(timeUnitAdd(unit, endTime))) {
                keys.add(DATE_FORMAT.format(startTime));
                Date finalStartTime = startTime;
                // 筛选已使用的ip 在指定时间前创建
                List<Ip> targetIps = ips.stream().filter(ip -> null != ip.getStatus() && !"free".equals(ip.getStatus().name()))
                        .filter(ip -> ip.getGmtCreate().before(finalStartTime) && null != ip.getPoolId())
                        .collect(Collectors.toList());
                Map<String, Ip> ipUsedMap = targetIps.stream().collect(Collectors.toMap(Ip::getIp, a -> a, (k1, k2) -> k1));
                // ip已使用
                Integer ipNum = ipUsedMap.size();
                usedData.add(ipNum);
                ArrayList<Integer> ipTimeData = new ArrayList<>(usedData);
                usedMap.put("name", "已使用(个)");
                usedMap.put("data", ipTimeData);
                // 使用率
                if (0 == ipTotal.doubleValue()) {
                    percentData.add(Double.valueOf(0));
                } else {
                    double rate = BigInteger.valueOf(targetIps.size()).doubleValue() / ipTotal.doubleValue();
                    percentData.add(Double.isNaN(rate) ? 0 : MathUtils.round(rate, 2));
                }
                ArrayList<Double> percentTimeData = new ArrayList<>(percentData);
                percentMap.put("name", "使用率(%)");
                percentMap.put("data", percentTimeData);
                startTime = timeUnitAdd(unit, startTime);
            }
            values.add(usedMap);
            values.add(percentMap);
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            dataResult.put("keys", timeKeys);
            dataResult.put("values", values);
            return new BocloudResult(true, dataResult, "查询成功");
        } catch (Exception e) {
            logger.error("get ip data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    private BocloudResult getComputerResource(Long vendorId, Long vdcId, Long clusterId, String start, String end,
                                              String unit, String serverName, String vendorType) {
        SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> dataResult = Maps.newHashMap();
        try {
            List<String> keys = new ArrayList<>();
            List<Map<String, Object>> values = new ArrayList<>();

            List<CloudServer> listCloudServers = new ArrayList<CloudServer>();
            if (null != clusterId && clusterId > 0) {
                BocloudResult cloudServerResult = imsInternalService.getVmList(null, null, clusterId);
                if (cloudServerResult.getData() != null) {
                    listCloudServers = JSONArray.parseArray(JSONObject.toJSONString(cloudServerResult.getData()),
                            CloudServer.class);
                }
            } else {
                listCloudServers = listCloudServers(null, Boolean.FALSE, null, vendorType);
            }
            if (null != vendorId) {
                // == --> equals
                listCloudServers = listCloudServers.stream().filter(cloudServer -> vendorId.equals(cloudServer.getVendorId())).collect(Collectors.toList());
            }

            Date endTime = DateUtils.stringToDate(end, "yyyy-MM-dd HH:mm:ss");
            Date startTime = DateUtils.stringToDate(start, "yyyy-MM-dd HH:mm:ss");
            List<Integer> cpuData = new ArrayList<>();
            List<Double> memData = new ArrayList<>();
            List<Integer> vmCountData = new ArrayList<>();
            Map<String, Object> cpuMap = Maps.newHashMap();
            Map<String, Object> memMap = Maps.newHashMap();
            Map<String, Object> vmCountMap = Maps.newHashMap();
            while (startTime.before(timeUnitAdd(unit, endTime))) {
                keys.add(DATE_FORMAT.format(startTime));
                Date finalStartTime = startTime;
                // 筛选目标虚机 在 指定时间前创建 指定条件主机下 非创建中的虚机
                List<CloudServer> targetVms = listCloudServers.stream()
                        .filter(cloudServer -> cloudServer.getGmtCreate().before(finalStartTime))
                        .collect(Collectors.toList());
                // cpu
                Integer cpuNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0)).reduce(Integer::sum)
                        .orElse(0);
                cpuData.add(cpuNum);
                ArrayList<Integer> cpuTimeData = new ArrayList<>(cpuData);
                cpuMap.put("name", "CPU(C)");
                cpuMap.put("data", cpuTimeData);
                // 内存
                BigDecimal memNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                memData.add(memNum.doubleValue());
                ArrayList<Double> memTimeData = new ArrayList<>(memData);
                memMap.put("name", "内存(GB)");
                memMap.put("data", memTimeData);
                // 虚机个数
                vmCountData.add(targetVms.size());
                ArrayList<Integer> vmCountTimeData = new ArrayList<>(vmCountData);
                vmCountMap.put("name", "虚机(个)");
                vmCountMap.put("data", vmCountTimeData);
                startTime = timeUnitAdd(unit, startTime);
            }
            values.add(cpuMap);
            values.add(memMap);
            values.add(vmCountMap);
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            dataResult.put("keys", timeKeys);
            dataResult.put("values", values);
            return new BocloudResult(true, dataResult, "查询成功");
        } catch (Exception e) {
            logger.error("get computer data resource error", e);
            return new BocloudResult(false, null, "查询失败");
        }
    }

    private Date timeUnitAdd(String unit, Date startTime) {
        if (TimeUnit.DAYS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addHours(startTime, 1);
        } else if (TimeUnit.WEEKS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addHours(startTime, 12);
        } else if (TimeUnit.WEEKDAYS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addDays(startTime, 1);
        } else if (TimeUnit.MONTHS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addDays(startTime, 1);
        } else if (TimeUnit.QUARTER_YEARS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addDays(startTime, 7);
        } else if (TimeUnit.HALF_YEARS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addWeeks(startTime, 1);
        } else if (TimeUnit.YEARS.getName().equals(unit)) {
            startTime = org.apache.commons.lang3.time.DateUtils.addMonths(startTime, 1);
        }
        return startTime;
    }
}
