package com.bocloud.rms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.rms.entity.PcBillTemplate;
import com.bocloud.rms.entity.PcTemplateProperty;
import com.bocloud.rms.intf.PublicCloudBillService;
import com.bocloud.rms.repository.PcBillTemplateRepository;
import com.bocloud.rms.repository.PcBillsRepository;
import com.bocloud.rms.repository.PcTemplatePropertyRepository;
import com.bocloud.rms.repository.PcTemplatePropertyValueRepository;
import com.bocloud.rms.service.utils.ImportHelper;
import com.bocloud.rms.service.utils.ResourceEventPublisher;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 * @version 1.0
 * @date 2021/2/22 10:50
 */
@Service("publicCloudBillService")
public class PublicCloudBillServiceImpl implements PublicCloudBillService {
    private static Logger logger = LoggerFactory.getLogger(PublicCloudBillServiceImpl.class);
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private PcBillTemplateRepository pcBillTemplateRepository;
    @Autowired
    private PcTemplatePropertyRepository pcTemplatePropertyRepository;
    @Autowired
    private PcTemplatePropertyValueRepository pcTemplatePropertyValueRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PcBillsRepository pcBillsRepository;
    @Value("${spring.datasource.druid.url}")
    private String dataType;

    @Override
    public BocloudResult templates(Pager pager, RequestContext context) {
        GridBean gridBean;
        int page = pager.getPage();
        int rows = pager.getRows();
        List<Param> params = pager.getParams();
        Map<String, String> sorter = pager.getSorter();
        boolean simple = pager.getSimple();
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            int count = pcBillTemplateRepository.count(params);
            List<PcBillTemplate> list = pcBillTemplateRepository.list(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, count, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult removeTemplate(Long id, RequestContext context) {
        String path = PcBillTemplate.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PcBillTemplate template = pcBillTemplateRepository.query(id);
            Assert.notNull(template, "数据不存在！");
            //删除账单模板
            pcBillTemplateRepository.delete(id, context.getTarget());
            //删除账单模板属性
            pcTemplatePropertyRepository.deleteByTemplateId(id, context.getTarget());
            //删除账单模板属性数据
            pcTemplatePropertyValueRepository.deleteByTemplateId(id, context.getTarget());
            return new BocloudResult(true, "删除公有云账单成功");
        } catch (Exception e) {
            logger.error("remove public cloud bill fail:", e);
            throw new InternalServerException("删除公有云账单失败," + e.getMessage(), e, e.getMessage());
        }
    }

    @Override
    public BocloudResult download(Long id, Pager pager, RequestContext context) {
        CsvWriter csvWriter = null;
        try {
            PcBillTemplate template = pcBillTemplateRepository.query(id);
            Assert.notNull(template, "公有云账单不存在");
            String name = template.getName();
            String source = "ALI".equals(template.getSource()) ? "阿里云" : "华为云";
            List<Param> params = pager.getParams();
            if (null == params) {
                params = new ArrayList<>();
            }
            Map<String, String> sorter = pager.getSorter();
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<Long> ids = Lists.newArrayList();
            if (null != template.getMerge()) {
                ids = JSONArray.parseArray(template.getMerge()).toJavaList(Long.class);
            } else {
                ids.add(id);
            }
            //表头选中状态加载
            Map<Long, List<PcTemplateProperty>> propertyMap = Maps.newHashMap();
            for (Long templateId : ids) {
                List<PcTemplateProperty> properties = pcTemplatePropertyRepository.queryByPropertyAll(templateId);
                propertyMap.put(templateId, properties);
            }
            List<Map<String, Object>> list = pcBillsRepository.list(1, Integer.MAX_VALUE, params, sorter, propertyMap, false, false, template.getSource(), dataType);
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            csvWriter = new CsvWriter(byteArrayOutputStream, ',', Charset.forName("GBK"));
            List<String> headers = Lists.newArrayList();
            for (int i = 0; i < list.size(); i++) {
                Map<String, Object> resultMap = list.get(i);
                List<String> content = Lists.newArrayList();
                for (String key : resultMap.keySet()) {
                    if (!"id".equals(key) && !"gmt_create".equals(key)) {
                        if (0 == i) {
                            headers.add(key);
                        }
                        content.add(String.valueOf(null == resultMap.get(key) ? "" : resultMap.get(key)));
                    }
                }
                if (0 == i) {
                    csvWriter.writeRecord(headers.toArray(new String[headers.size()]));
                }
                csvWriter.writeRecord(content.toArray(new String[content.size()]), true);
            }
            csvWriter.close();
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", source + name.replaceAll(",", "\\/") + "账单" + System.currentTimeMillis() + ".csv");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("公有云下载失败", e);
            return new BocloudResult(false, "公有云下载失败");
        } finally {
            if (null != csvWriter) {
                csvWriter.close();
            }
        }
    }

    @Override
    public BocloudResult upload(MultipartFile file, Long contextId, String instance, String params) throws IOException {
        String uuid = IDFactory.instance().uuid();
        //借鉴https://ask.csdn.net/questions/868861
        //原理暂不清楚，模板会取到GBK，公有云账单会取UTF-8
        InputStream inputStream = file.getInputStream();
        int p = (inputStream.read() << 8) + inputStream.read();
        String code = null;

        switch (p) {
            case 0xefbb:
                code = "UTF-8";
                break;
            case 0xfffe:
                code = "Unicode";
                break;
            case 0xfeff:
                code = "UTF-16BE";
                break;
            default:
                code = "GBK";
        }

        CsvReader csvReader = new CsvReader(inputStream, Charset.forName(code));
        try {
            JSONObject object = JSONObject.parseObject(params);
            String name = object.getLong("year") + "-" + object.getLong("month");
            PcBillTemplate pcBillTemplate = pcBillTemplateRepository.queryByName(name, object.getString("source"));
            if (null != pcBillTemplate) {
                return new BocloudResult(false, "当前导入账单已存在,请检查！");
            }
            taskExecutor.execute(() -> {
                ImportHelper importHelper = new ImportHelper(pcBillTemplateRepository, pcTemplatePropertyRepository,
                        pcTemplatePropertyValueRepository, resourceEventPublisher);
                importHelper.readCsv(csvReader, uuid, contextId, object);
            });
        } catch (Exception e) {
            logger.error("公有云账单导入失败", e);
            return new BocloudResult(false, "公有云账单导入失败");
        }
        return new BocloudResult(true, uuid, "公有云账单导入任务已下发");
    }

    @Override
    public BocloudResult detail(Long id, Pager pager, RequestContext context) {
        logger.info("dataType：" + dataType);
        GridBean gridBean;
        int page = pager.getPage();
        int rows = pager.getRows();
        List<Param> params = pager.getParams();
        Map<String, String> sorter = pager.getSorter();
        boolean simple = pager.getSimple();
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            PcBillTemplate template = pcBillTemplateRepository.query(id);
            Assert.notNull(template, "公有云账单不存在");
            String merge = template.getMerge();
            List<Long> ids = Lists.newArrayList();
            if (null != merge) {
                ids = JSONArray.parseArray(template.getMerge()).toJavaList(Long.class);
            } else {
                ids.add(id);
            }
            //表头选中状态加载
            List<PcTemplateProperty> propertiesAll = pcTemplatePropertyRepository.queryByPropertyAll(ids.get(0));
            Map<String, String> transitionMap = Maps.newHashMap();
            for (PcTemplateProperty property : propertiesAll) {
                transitionMap.put(property.getName(), property.getCode());
            }
            //表头加载状态的数据
            List<String> names = Lists.newArrayList();
            Map<Long, List<PcTemplateProperty>> propertyMap = Maps.newHashMap();
            for (int i = 0; i < ids.size(); i++) {
                Long templateId = ids.get(i);
                if (0 == i) {
                    List<PcTemplateProperty> properties = pcTemplatePropertyRepository.queryByPropertyStatus(templateId);
                    for (PcTemplateProperty property : properties) {
                        names.add(property.getName());
                    }
                    propertyMap.put(templateId, properties);
                } else {
                    List<PcTemplateProperty> properties = pcTemplatePropertyRepository.query(templateId, names);
                    propertyMap.put(templateId, properties);
                }
            }
            int total = pcBillsRepository.count(params, propertyMap, template.getSource(), dataType);
            List<Map<String, Object>> list = pcBillsRepository.list(page, rows, params, sorter, propertyMap, simple, false, template.getSource(), dataType);
            List<Map<String, Object>> result = Lists.newArrayList();
            for (Map<String, Object> map : list) {
                HashMap<String, Object> resultMap = Maps.newHashMap();
                for (String key : map.keySet()) {
                    if ("id".equals(key.toLowerCase()) || "gmt_create".equals(key.toLowerCase())) {
                        resultMap.put(key.toLowerCase(), map.get(key));
                    } else {
                        resultMap.put(transitionMap.get(key.toLowerCase()), map.get(key));
                    }
                }
                result.add(resultMap);
            }
            gridBean = GridHelper.getBean(page, rows, total, result);
            Map<String, Object> data = Maps.newHashMap();
            data.put("page", gridBean.getPage());
            data.put("rows", gridBean.getRows());
            data.put("total", gridBean.getTotal());
            data.put("pages", gridBean.getPages());
            data.put("columns", JSONObject.toJSON(propertiesAll));
            return new BocloudResult(true, data, "查询成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult columns(Long propertyId, Boolean checked, RequestContext context) {
        try {
            PcTemplateProperty property = pcTemplatePropertyRepository.query(propertyId);
            Assert.notNull(property, "账单属性不存在");
            property.setShow(checked);
            property.setGmtModify(new Date());
            property.setMenderId(context.getTarget());
            pcTemplatePropertyRepository.update(property);
            return new BocloudResult(true, "更新成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "更新失败");
        }
    }

    @Override
    public BocloudResult combine(List<Long> combine, RequestContext context) {
        try {
            String year = null;
            String months = "";
            String source = null;
            List<Long> templates = Lists.newArrayList();
            Map<Long, PcBillTemplate> checkMap = Maps.newHashMap();
            for (int i = 0; i < combine.size(); i++) {
                PcBillTemplate template = pcBillTemplateRepository.query(combine.get(i));
                if (checkMap.containsKey(template.getId())) {
                    continue;
                }
                checkMap.put(template.getId(), template);
                String merge = template.getMerge();
                if (0 == i) {
                    source = template.getSource();
                    year = StringUtils.substringBefore(template.getName(), "-") + "-";
                } else {
                    if (!StringUtils.substringBefore(year, "-").equals(StringUtils.substringBefore(template.getName(),
                            "-"))) {
                        return new BocloudResult(false, "不同年份账单暂不支持合并");
                    }
                    if (!template.getSource().equals(source)) {
                        return new BocloudResult(false, "不同公有云账单暂不支持合并");
                    }
                }
                if (null != merge) {
                    List<Long> ids = JSONArray.parseArray(merge).toJavaList(Long.class);
                    for (Long id : ids) {
                        if (checkMap.containsKey(id)) {
                            continue;
                        }
                        templates.add(id);
                        PcBillTemplate pcBillTemplate = pcBillTemplateRepository.query(id);
                        months += (pcBillTemplate.getMonth() + ",");
                        year += StringUtils.substringBefore(pcBillTemplate.getMonth(), "月") + ",";
                        checkMap.put(id, pcBillTemplate);
                    }
                } else {
                    templates.add(template.getId());
                    months += (template.getMonth() + ",");
                    year += StringUtils.substringBefore(template.getMonth(), "月") + ",";
                }
            }
            PcBillTemplate template = new PcBillTemplate();
            template.setName(StringUtils.substringBeforeLast(year, ","));
            template.setMonth(StringUtils.substringBeforeLast(months, ","));
            template.setGmtCreate(new Date());
            template.setCreatorId(context.getTarget());
            template.setDeleted(false);
            template.setMerge(String.valueOf(templates));
            template.setSource(source);
            pcBillTemplateRepository.save(template);
//            //保存合并账单列属性
//            List<PcTemplateProperty> properties = pcTemplatePropertyRepository.queryByTemplateId(combine.get(0));
//            for (PcTemplateProperty property : properties) {
//                String value = property.getName();
//                property.setId(null);
//                property.setCreatorId(context.getTarget());
//                property.setGmtCreate(new Date());
//                property.setMenderId(null);
//                property.setGmtModify(null);
//                property.setTemplateId(template.getId());
//                if ("账期".equals(value) || "产品".equals(value) || "计费模式".equals(value) || "产品类型".equals(value) || "服务时长".equals(value)
//                        || "时长单位".equals(value) || "账单类型".equals(value) || "".equals(value) || "资源名称".equals(value) || "实例昵称".equals(value)
//                        || "资源标签".equals(value) || "实例标签".equals(value) || "规格".equals(value) || "实例配置".equals(value) || "区域".equals(value)
//                        || "地域".equals(value) || "官网价(￥)".equals(value) || "原价".equals(value)) {
//                    property.setShow(true);
//                } else {
//                    property.setShow(false);
//                }
//            }
//            pcTemplatePropertyRepository.batchSave(properties, 1000);
            return new BocloudResult(true, "账单合并成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "账单合并失败");
        }
    }

    @Override
    public BocloudResult detailSelect(Long id, RequestContext context) {
        logger.info("dataType：" + dataType);
        try {
            Pager pager = new Pager(1, Integer.MAX_VALUE, null, null);
            Map<String, Set<String>> result = new HashMap<>();
            BocloudResult detail = this.detail(id, pager, context);
            if (detail.isSuccess()) {
                List<JSONObject> columns = JSONArray.parseArray(String.valueOf(JSONObject.parseObject(JSONObject.toJSONString(detail.getData())).get("columns")), JSONObject.class);
                Map<String, String> columsMap = new HashMap<>();
                columns.stream().forEach(
                        column -> {
                            if (column.get("name").equals("账号") || column.get("name").equals("产品") || column.get("name").equals("产品明细") || column.get("name").equals("账单类型") || column.get("name").equals("产品类型")) {
                                columsMap.put(column.get("name").toString(), column.get("code").toString());
                            }
                        }
                );
                if (columsMap.size() == 0 || columsMap.isEmpty()) {
                    return new BocloudResult(true, result, "此账单没有列对应信息");
                }
                List<JSONObject> rows = JSONArray.parseArray(String.valueOf(JSONObject.parseObject(JSONObject.toJSONString(detail.getData())).get("rows")), JSONObject.class);
                for (String key : columsMap.keySet()) {
                    result.put(key, new HashSet<>());
                }
                for (JSONObject row : rows) {
                    for (String key : columsMap.keySet()) {
                        columsMap.get(key);
                        if (null != row.get(columsMap.get(key))) {
                            result.get(key).add(row.get(columsMap.get(key)).toString());
                        }
                    }
                }
                return new BocloudResult(true, result, "公有云账单详情选择条件查询成功");
            } else {
                return detail;
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "公有云账单详情选择条件查询失败");
        }


    }
}

