package com.bocloud.rms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.rms.internel.entity.CloudServer;
import com.bocloud.rms.enums.CloudProviderType;
import com.bocloud.rms.intf.ReportService;
import com.bocloud.rms.repository.ReportRepository;
import com.csvreader.CsvWriter;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.nio.charset.Charset;
import java.util.*;

/**
 * @author lyy
 * @version 1.0
 * @date 2021/3/15 17:14
 */
@Service("reportService")
public class ReportServiceImpl implements ReportService {
    private static Logger logger = LoggerFactory.getLogger(ReportServiceImpl.class);
    @Autowired
    private ReportRepository reportRepository;
    @Value(("${report.cache.file:/reports/}"))
    private String filePath;

    @Override
    public BocloudResult vmList(Pager pager, RequestContext context) {
        List<CloudServer> list;
        int page = pager.getPage();
        int rows = pager.getRows();
        List<Param> params = pager.getParams();
        Map<String, String> sorter = pager.getSorter();
        boolean simple = pager.getSimple();
        int total;
        GridBean gridBean;
        try {
            params = null == params ? new ArrayList<>() : params;
            sorter = null == sorter ? new HashMap<>() : sorter;
            sorter.put("gmtCreate", Common.ONE);
            total = reportRepository.vmList(1, Integer.MAX_VALUE, params, sorter).size();
            list = reportRepository.vmList(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult vmsDownload(Pager pager, RequestContext context) {
        CsvWriter csvWriter = null;
        try {
            List<Param> params = pager.getParams();
            Map<String, String> sorter = pager.getSorter();
            List<CloudServer> cloudServers = reportRepository.vmList(1, Integer.MAX_VALUE, null == params ? new ArrayList<>()
                    : params, null == sorter ? new HashMap<>() : sorter);
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            csvWriter = new CsvWriter(byteArrayOutputStream, ',', Charset.forName("GBK"));
            //标题
            String[] headers = {"虚拟机名称", "虚拟机类型", "IP地址", "配置信息", "租户", "项目", "申请人", "使用人", "所属平台"};
            csvWriter.writeRecord(headers);
            //内容
            for (CloudServer cloudServer : cloudServers) {
                List<String> content = Lists.newArrayList();
                content.add(cloudServer.getName());
                content.add(cloudServer.getVendorType());
                if (CloudProviderType.OPENSTACK.name().equalsIgnoreCase(cloudServer.getVendorType()) || "MANAGEONE".equalsIgnoreCase(cloudServer.getVendorType())) {
                    JSONArray privateIps = JSONArray.parseArray(cloudServer.getPrivateIps());
                    if (null != privateIps) {
                        String ipString = "";
                        List<JSONObject> objects = privateIps.toJavaList(JSONObject.class);
                        if (!ListTool.isEmpty(objects)) {
                            for (JSONObject object : objects) {
                                List<Object> addresses = object.getJSONArray("addresses");
                                for (Object address : addresses) {
                                    if (StringUtils.isEmpty(JSONObject.parseObject(JSONObject.toJSONString(address))
                                            .getString("address"))) {
                                        continue;
                                    }
                                    ipString += JSONObject.parseObject(JSONObject.toJSONString(address)).getString(
                                            "address")
                                            + ",";
                                }
                            }
                        }
                        if (ipString.length() > 0) {
                            ipString = ipString.substring(0, ipString.length() - 1);
                        }
                        content.add(ipString);
                    } else {
                        content.add("");
                    }
                } else if (CloudProviderType.TENCENT.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<String> privateIpLists = JSONObject.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(privateIpLists)) {
                        for (String object : privateIpLists) {
                            privateIpString += object + ",";
                        }
                        if (privateIpString.length() > 0) {
                            privateIpString = privateIpString.substring(0, privateIpString.length() - 1);
                        }
                    }
                    content.add(privateIpString);
                } else if (CloudProviderType.HUAWEI.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ips = "";
                    List<JSONObject> parseArray = JSON.parseArray(privateIps, JSONObject.class);
                    if (null != parseArray && parseArray.size() > 0) {
                        for (JSONObject jsonObject : parseArray) {
                            String address = jsonObject.get("address").toString();
                            ips += address + ",";
                        }
                    }
                    content.add(ips);
                } else if (CloudProviderType.TIANYI.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ips = "";
                    List<String> parseArray = JSON.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(parseArray)) {
                        for (String ip : parseArray) {
                            ips += ip + ",";
                        }
                    }
                    if (ips.length() > 0) {
                        ips = ips.substring(0, ips.length() - 1);
                    }
                    content.add(ips);
                } else if (CloudProviderType.QCLOUD.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<JSONObject> jSONObjectLists = JSONObject.parseArray(privateIps, JSONObject.class);
                    if (!ListTool.isEmpty(jSONObjectLists)) {
                        JSONObject jsonObject = jSONObjectLists.get(0);
                        if (null != jsonObject) {
                            privateIpString = jsonObject.getString("address");
                        }
                    }
                    content.add(privateIpString);
                } else if (CloudProviderType.VMWARE.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ipString = "";
                    if (!StringUtils.isEmpty(privateIps)) {
                        List<JSONObject> objects = JSONObject.parseArray(privateIps, JSONObject.class);
                        for (JSONObject object : objects) {
                            String address = object.getString("address");
                            if (StringUtils.isEmpty(address)) {
                                continue;
                            }
                            ipString += address + ",";
                        }
                    }
                    if (ipString.length() > 0) {
                        ipString = ipString.substring(0, ipString.length() - 1);
                    }
                    content.add(ipString);
                } else if (CloudProviderType.AWS.name().equalsIgnoreCase(cloudServer.getVendorType())
                        || CloudProviderType.ALIYUN.name().equalsIgnoreCase(cloudServer.getVendorType())
                        || CloudProviderType.AZURE.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<String> privateIpLists = JSONObject.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(privateIpLists)) {
                        for (String object : privateIpLists) {
                            privateIpString += object + ",";
                        }
                        if (privateIpString.length() > 0) {
                            privateIpString = privateIpString.substring(0, privateIpString.length() - 1);
                        }
                    }
                    content.add(privateIpString);
                } else {
                    content.add(cloudServer.getPrivateIps());
                }

                StringBuilder specification = new StringBuilder();
                content.add(specification.append(cloudServer.getCpu()).append("C/").append(cloudServer.getMemory()).append("GB/")
                        .append(cloudServer.getDisk()).append("GB").toString());
                content.add(cloudServer.getTenantName());
                content.add(cloudServer.getProjectName());
                content.add(cloudServer.getTargetName());
                content.add(cloudServer.getOwnerName());
                content.add(cloudServer.getVendorName());
                csvWriter.writeRecord(content.toArray(new String[content.size()]));
            }
            csvWriter.close();
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", "虚拟机清单报表" + System.currentTimeMillis() + ".csv");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("虚拟机清单报表下载失败", e);
            return new BocloudResult(false, "虚拟机清单报表下载失败");
        } finally {
            if (null != csvWriter) {
                csvWriter.close();
            }
        }
    }

}
