package com.bocloud.rms.service;

import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.internel.entity.Project;
import com.bocloud.rms.entity.CloudService;
import com.bocloud.rms.entity.ServiceBill;
import com.bocloud.rms.entity.ServiceOrder;
import com.bocloud.rms.intf.ServiceBillService;
import com.bocloud.rms.repository.CloudServiceRepository;
import com.bocloud.rms.repository.ServiceBillRepository;
import com.bocloud.rms.repository.ServiceOrderRepository;
import com.bocloud.rms.service.internal.SmsProjectService;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 账单业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年6月12日
 */
@Service("serviceBillService")
public class ServiceBillServiceImpl implements ServiceBillService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceBillServiceImpl.class);
    private final ServiceBillRepository billRepository;
    private final ServiceOrderRepository orderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final SmsProjectService smsProjectService;


    public ServiceBillServiceImpl(ServiceOrderRepository orderRepository, ServiceBillRepository billRepository,
                                   SmsProjectService smsProjectService,
                                  CloudServiceRepository cloudServiceRepository
                                  ) {
        this.orderRepository = orderRepository;
        this.billRepository = billRepository;
        this.smsProjectService = smsProjectService;
        this.cloudServiceRepository = cloudServiceRepository;
    }

    @Override
    public BocloudResult<GridBean<ServiceBill>> list(Pager pager, RequestContext context) {
        try {
            List<ServiceBill> list;
            int total;
            GridBean<ServiceBill> gridBean;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            String serviceCode = null;
            String orderSn = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("serviceCode")) {
                    serviceCode = paramMap.get("serviceCode").toString();
                    paramMap.remove("serviceCode");
                }
                if (paramMap.containsKey("orderSn")) {
                    orderSn = paramMap.get("orderSn").toString();
                    paramMap.remove("orderSn");
                }
            }
            List<Long> projectIds = new ArrayList<>();
            if (Catalog.Tenant == context.getCatalog()) {
                // 租户能看到 1.属于当前租户的订单
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                total = billRepository.count(params, serviceCode, orderSn, projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, projectIds);
            } else if (Catalog.User == context.getCatalog()) {
                // 用户能看到 1.属于当前用户的订单 2.当前用户管理的项目的订单
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                total = billRepository.count(params, serviceCode, orderSn, context.getTarget(), projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, context.getTarget(),
                                projectIds);
            } else {
                total = billRepository.count(params, serviceCode, orderSn, projectIds);
                list = billRepository
                        .list(pager.getPage(), pager.getRows(), sorter, params, serviceCode, orderSn, projectIds);
            }
            for (ServiceBill bill : list) {
                ServiceOrder order = orderRepository.query(bill.getOrderId());
                if (null != order) {
                    CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
                    if (null != service) {
                        bill.setServiceName(service.getName());
                    }
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

}