package com.bocloud.rms.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.internel.entity.CloudServer;
import com.bocloud.rms.internel.entity.Cluster;
import com.bocloud.rms.internel.entity.Server;
import com.bocloud.rms.service.utils.CaptchaCache;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * cmp内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月9日
 */
@Component("imsInternalService")
public class ImsInternalService implements InitializingBean {

    private static Logger logger = LoggerFactory.getLogger(ImsInternalService.class);

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private CaptchaCache captchaCache;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create(BoCloudService.IMS);
    private List<CloudServer> cloudServers = null;
    private LocalDateTime requestVmTime = null;
    private List<Server> servers = null;
    private LocalDateTime requestServerTime = null;
    private List<Cluster> clusters = null;
    private LocalDateTime requestClusetrTime = null;

    /**
     * 获取主机列表
     *
     * @return
     */
    public BocloudResult getHostList(Long vendorId, Long vdcId, Long clusterId, String name, String vendorType) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vendorType", vendorType);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        jsonObject.put("name", name);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/hosts/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取虚机列表
     *
     * @return
     */
    public BocloudResult getVmList(Long vendorId, Long vdcId, Long clusterId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取虚机列表
     *
     * @param serverIds
     * @return
     */
    public BocloudResult getVmList(List<Long> serverIds) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByVms");
        jsonObject.put("serverIds", serverIds);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取虚机列表
     *
     * @param tenantIds
     * @return
     */
    public List<CloudServer> listVms(List<Long> tenantIds, Boolean isAll, List<Long> projectIds) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listTenantProject");
        if (!ListTool.isEmpty(tenantIds)) {
            jsonObject.put("tenantIds", tenantIds);
        }
        if (!ListTool.isEmpty(projectIds)) {
            jsonObject.put("projectIds", projectIds);
        }
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        BocloudResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        cloudServers = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        requestVmTime = LocalDateTime.now();
        return cloudServers;
    }


    /**
     * 获取已使用Ip
     *
     * @param poolName
     * @return
     */
    public BocloudResult getIpNumber(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        if (poolName != null) {
            params.add(new Param(MapTools.simpleMap("poolName", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ips", BoCloudMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

    /**
     * 获取ip池列表
     *
     * @param poolName
     * @return
     */
    public BocloudResult getIpPoolList(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("countIp", true), Sign.EQ));
        if (null != poolName) {
            params.add(new Param(MapTools.simpleMap("name", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ip/pools", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
