package com.bocloud.rms.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.rms.internel.entity.Project;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Component
public class SmsProjectService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsProjectService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create(BoCloudService.SMS);


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    /**
     * 查询用户管理的项目
     *
     * @param userId
     * @return
     */
    public List<Project> listUserManagerProjects(Long userId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listUserManagerProjects");
        object.put("userId", userId);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects/condition", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

}
