package com.bocloud.rms.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.rms.internel.entity.Tenant;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * soc服务租户内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月4日
 */
@Component
public class SmsTenantService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsTenantService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create(BoCloudService.SMS);
    private final Map<String, Object> headers = new HashMap<>(4);

    /**
     * 分页获取用户列表
     *
     * @param pager 分页对象
     * @return
     */
    public List<Tenant> list(Pager pager) {
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/tenants",
                BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult<?> result = remoteService.invoke();
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(Tenant.class);
    }


    /**
     * 获取所有一级租户(分页)
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     */
    public BocloudResult listParentTenant(int page, int rows, List<Param> params, Map<String, String> sorter) {
        Pager pager = new Pager(page, rows, params, sorter);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants",
                BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Assert.notNull(result.getData(), "获取租户信息失败");
        return result;
    }

    public BocloudResult<List<Tenant>> listChildren(Long parentId) {
        Map<String, Object> params = Maps.newHashMap();
        params.put("parentId", parentId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + parentId + "/children",
                BoCloudMethod.GET, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONArray.parseArray(data, Tenant.class), "查询成功");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

}
