package com.bocloud.rms.service.utils;

import com.bocloud.common.utils.ListTool;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: CsvExportUtil
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/19 15:17
 */
public class CsvExportUtil {

    /**
     * CSV文件列分隔符
     */
    private static final String CSV_COLUMN_SEPARATOR = ",";

    /**
     * CSV文件行分隔符
     */
    private static final String CSV_ROW_SEPARATOR = "\r\n";

    /**
     * @param dataList 集合数据
     * @param titles   表头部数据
     * @param keys     表内容的键值
     * @param os       输出流
     */
    public static void doExport(List<Map<String, Object>> dataList, String titles, String keys, OutputStream os) throws Exception {

        // 保证线程安全
        StringBuffer buf = new StringBuffer();

        String[] titleArr = null;
        String[] keyArr = null;

        titleArr = titles.split(",");
        keyArr = keys.split(",");

        // 组装表头
        for (String title : titleArr) {
            buf.append(title).append(CSV_COLUMN_SEPARATOR);
        }
        buf.append(CSV_ROW_SEPARATOR);

        // 组装数据
        if (!ListTool.isEmpty(dataList)) {
            for (Map<String, Object> data : dataList) {
                for (String key : keyArr) {
                    buf.append(data.get(key)).append(CSV_COLUMN_SEPARATOR);
                }
                buf.append(CSV_ROW_SEPARATOR);
            }
        }
        // 写出响应
        byte[] uft8bom = {(byte) 0xef, (byte) 0xbb, (byte) 0xbf};
        os.write(uft8bom);
        os.write(buf.toString().getBytes("UTF-8"));
        os.flush();
    }

    public static void doTitle(String titles, OutputStream os) throws Exception {
        // 保证线程安全
        StringBuffer buf = new StringBuffer();
        String[] titleArr = null;
        titleArr = titles.split(",");
        // 组装表头
        for (String title : titleArr) {
            buf.append(title).append(CSV_COLUMN_SEPARATOR);
        }
        buf.append(CSV_ROW_SEPARATOR);
        // 写出响应
        byte[] uft8bom = {(byte) 0xef, (byte) 0xbb, (byte) 0xbf};
        os.write(uft8bom);
        os.write(buf.toString().getBytes("UTF-8"));
    }

    public static void doExport(List<Map<String, Object>> dataList, String keys, OutputStream os) throws Exception {
        // 保证线程安全
        StringBuffer buf = new StringBuffer();
        String[] keyArr = keys.split(",");
        // 组装数据
        if (!ListTool.isEmpty(dataList)) {
            for (Map<String, Object> data : dataList) {
                for (String key : keyArr) {
                    buf.append(data.get(key)).append(CSV_COLUMN_SEPARATOR);
                }
                buf.append(CSV_ROW_SEPARATOR);
            }
        }
        // 写出响应
        byte[] uft8bom = {(byte) 0xef, (byte) 0xbb, (byte) 0xbf};
        os.write(uft8bom);
        os.write(buf.toString().getBytes("UTF-8"));
    }

    /**
     * 设置Header
     *
     * @param fileName
     * @param response
     * @throws
     */
    public static void responseSetProperties(String fileName, HttpServletResponse response) throws UnsupportedEncodingException {
        // 设置文件后缀
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
//        String fn = fileName + sdf.format(new Date()) + ".csv";
        String fn = fileName + System.currentTimeMillis() + ".csv";
        // 读取字符编码
        String utf = "UTF-8";

        // 设置响应
        response.setContentType("application/ms-txt.numberformat:@");
        response.setCharacterEncoding(utf);
        response.setHeader("Pragma", "public");
        response.setHeader("Cache-Control", "max-age=30");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fn, utf));
    }

}
