package com.bocloud.rms.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.rms.entity.PcBillTemplate;
import com.bocloud.rms.entity.PcTemplateProperty;
import com.bocloud.rms.entity.PcTemplatePropertyValue;
import com.bocloud.rms.repository.PcBillTemplateRepository;
import com.bocloud.rms.repository.PcTemplatePropertyRepository;
import com.bocloud.rms.repository.PcTemplatePropertyValueRepository;
import com.csvreader.CsvReader;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 导入数据处理
 *
 * @author liuyuanyuan
 */
public class ImportHelper {
    private static Logger logger = LoggerFactory.getLogger(ImportHelper.class);
    private PcBillTemplateRepository pcBillTemplateRepository;
    private PcTemplatePropertyRepository pcTemplatePropertyRepository;
    private PcTemplatePropertyValueRepository pcTemplatePropertyValueRepository;
    private ResourceEventPublisher resourceEventPublisher;

    public ImportHelper() {
        super();
    }

    public ImportHelper(PcBillTemplateRepository pcBillTemplateRepository,
                        PcTemplatePropertyRepository pcTemplatePropertyRepository, PcTemplatePropertyValueRepository pcTemplatePropertyValueRepository,
                        ResourceEventPublisher resourceEventPublisher) {
        super();
        this.pcBillTemplateRepository = pcBillTemplateRepository;
        this.pcTemplatePropertyRepository = pcTemplatePropertyRepository;
        this.pcTemplatePropertyValueRepository = pcTemplatePropertyValueRepository;
        this.resourceEventPublisher = resourceEventPublisher;
    }

    public void readCsv(CsvReader csvReader, String uuid, Long userId, JSONObject object) {
        Long templateId = null;
        try {
            //标题
            Map<Integer, String> header = Maps.newHashMap();
            //数据
            Map<Integer, List<JSONObject>> data = Maps.newHashMap();
            int num = 1;
            while (csvReader.readRecord()) {
                // 读一整行
                String rawRecord = csvReader.getRawRecord();
                String[] split = rawRecord.split(",");
                //标题数据
                if (1 == num) {
                    for (int i = 1; i <= split.length; i++) {
                        String s = split[i - 1];
                        if (s.contains("账期") && s.length() >= 2) {
                            s = s.substring(s.length() - 2);
                        }
                        if (s.contains("期") && s.length() < 2) {
                            s = "账期";
                        }
                        if (s.contains("¥")) {
                            s = s.replaceAll("¥", "￥");
                        }
                        header.put(i, s);
                    }
                } else {
                    //数据内容
                    List<JSONObject> values = Lists.newArrayList();
                    for (int i = 0; i < split.length; i++) {
                        JSONObject rowData = new JSONObject();
                        rowData.put("value", split[i]);
                        rowData.put("column", i);
                        rowData.put("name", header.get(i + 1));
                        values.add(rowData);
                    }
                    data.put(num - 1, values);
                }
                num++;
            }
            //推送进度10%
            processMessage(uuid, userId, 1, 10, uuid, true);
            //保存账单模板
            PcBillTemplate template = new PcBillTemplate();
            template.setName(object.getLong("year") + "-" + object.getLong("month"));
            template.setMonth(object.getLong("month") + "月份");
            template.setSource(object.getString("source"));
            template.setGmtCreate(new Date());
            template.setCreatorId(userId);
            template.setDeleted(false);
            pcBillTemplateRepository.save(template);
            templateId = template.getId();
            //推送进度30%
            processMessage(uuid, userId, 1, 30, uuid, true);
            //保存账单属性
            List<PcTemplateProperty> properties = Lists.newArrayList();
            for (String value : header.values()) {
                PcTemplateProperty property = new PcTemplateProperty();
                property.setTemplateId(templateId);
                property.setName(value);
                if ("账期".equals(value) || "账号".equals(value) || "产品".equals(value) || "产品明细".equals(value) || "计费模式".equals(value) || "产品类型".equals(value) || "服务时长".equals(value)
                        || "时长单位".equals(value) || "账单类型".equals(value) || "".equals(value) || "资源名称".equals(value) || "实例昵称".equals(value)
                        || "资源标签".equals(value) || "实例标签".equals(value) || "规格".equals(value) || "实例配置".equals(value) || "区域".equals(value)
                        || "地域".equals(value) || "官网价(￥)".equals(value) || "原价".equals(value)) {
                    property.setShow(true);
                } else {
                    property.setShow(false);
                }
                property.setGmtCreate(new Date());
                property.setCreatorId(userId);
                property.setDeleted(false);
                property.setCode(IDFactory.instance().uuid());
                properties.add(property);
            }
            pcTemplatePropertyRepository.batchSave(properties, 1000);
            //推送进度60%
            processMessage(uuid, userId, 1, 60, uuid, true);
            //保存账单属性值数据
            List<PcTemplateProperty> templateProperties = pcTemplatePropertyRepository.queryByTemplateId(template.getId());
            Map<String, PcTemplateProperty> map = Maps.newHashMap();
            for (PcTemplateProperty templateProperty : templateProperties) {
                map.put(templateProperty.getName(), templateProperty);
            }
            List<PcTemplatePropertyValue> propertValues = Lists.newArrayList();
            //数据分组
            for (Integer key : data.keySet()) {
                List<JSONObject> values = data.get(key);
                String groupId = IDFactory.instance().uuid();
                for (JSONObject value : values) {
                    String valueData = value.getString("value");
                    //华为云去掉内容数据中的“”
                    valueData = valueData.replaceAll("\"", "");
                    String name = value.getString("name");
                    Integer column = value.getInteger("column");
                    PcTemplatePropertyValue propertyValue = new PcTemplatePropertyValue();
                    propertyValue.setTemplateId(templateId);
                    propertyValue.setPropertyId(map.get(name).getId());
                    propertyValue.setPropertyValue(valueData);
                    propertyValue.setGroupId(groupId);
                    propertyValue.setLine(Long.valueOf(column));
                    propertyValue.setGmtCreate(new Date());
                    propertyValue.setCreatorId(userId);
                    propertyValue.setDeleted(false);
                    propertValues.add(propertyValue);
                }
            }
            pcTemplatePropertyValueRepository.batchSave(propertValues, 1000);
            //推送进度100%
            processMessage(uuid, userId, 1, 100, uuid, true);
            logger.info("导入完成！");
        } catch (Exception e) {
            //数据回滚
            try {
                pcBillTemplateRepository.delete(templateId, userId);
                pcTemplatePropertyRepository.deleteByTemplateId(templateId, userId);
                pcTemplatePropertyValueRepository.deleteByTemplateId(templateId, userId);
            } catch (Exception exception) {
                logger.info("回滚数据失败", e);
            }
            processMessage(uuid, userId, 1, 0, uuid, false);
            logger.info("导入错误", e);
        } finally {
            csvReader.close();
        }
    }

    /**
     * 处理excel
     *
     * @param wb
     * @param uuid
     * @param userId
     * @throws Exception
     */
    public void readExcel(Workbook wb, String uuid, Long userId, String params) {
        int total = countDataNum(wb);
        //数据为空
        if (0 == total) {
            processMessage(uuid, userId, total, 60, uuid, false);
            return;
        }
        Long templateId = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // 遍历sheet页
        for (int n = 0; n < wb.getNumberOfSheets(); n++) {
            Sheet sheet = wb.getSheetAt(n);
            try {
                // 标题属性
                Map<Integer, String> header = ExcelHelper.readHeader4NameKey(sheet.getRow(0));
                //读取数据整合
                Map<Integer, List<JSONObject>> data = Maps.newHashMap();
                // 表头单元格开始位置，依次每行按照其索引位置开始读取
                int startIndex = 0;
                // 最后一列数
                int lastColumn = header.size() + startIndex;
                // 遍历行
                for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                    Row row = sheet.getRow(i);
                    if (row == null) {
                        throw new Exception("请检查第" + (i + 1) + "行下面是否有干扰数据。");
                    }
                    List<JSONObject> values = Lists.newArrayList();
                    for (int j = startIndex; j < lastColumn; j++) {
                        JSONObject rowData = new JSONObject();
                        String value;
                        //获取单元格的数据
                        Cell cell = row.getCell(j);
                        if (cell == null) {
                            value = "";
                        } else {
                            value = BoStringUtils.toString(cell.getRichStringCellValue());
                        }
                        rowData.put("value", value);
                        rowData.put("column", j);
                        rowData.put("name", header.get(j));
                        values.add(rowData);
                    }
                    data.put(i, values);
                }
                JSONObject object = JSONObject.parseObject(params);
                //推送进度10%
                processMessage(uuid, userId, total, 10, uuid, false);
                //保存账单模板
                PcBillTemplate template = new PcBillTemplate();
                template.setName(object.getString("name"));
                template.setMonth(object.getLong("month") + "月份");
                template.setGmtCreate(new Date());
                template.setCreatorId(userId);
                template.setDeleted(false);
                pcBillTemplateRepository.save(template);
                templateId = template.getId();
                //推送进度30%
                processMessage(uuid, userId, total, 30, uuid, false);
                //保存账单属性
                List<PcTemplateProperty> properties = Lists.newArrayList();
                for (String value : header.values()) {
                    PcTemplateProperty property = new PcTemplateProperty();
                    property.setTemplateId(templateId);
                    property.setName(value);
                    if ("账期".equals(value) || "产品".equals(value) || "计费模式".equals(value) || "产品类型".equals(value) || "服务时长".equals(value)
                            || "时长单位".equals(value) || "账单类型".equals(value) || "".equals(value) || "资源名称".equals(value) || "实例昵称".equals(value)
                            || "资源标签".equals(value) || "实例标签".equals(value) || "规格".equals(value) || "实例配置".equals(value) || "区域".equals(value)
                            || "地域".equals(value) || "官网价(￥)".equals(value) || "原价".equals(value)) {
                        property.setShow(true);
                    } else {
                        property.setShow(false);
                    }
                    property.setGmtCreate(new Date());
                    property.setCreatorId(userId);
                    property.setDeleted(false);
                    properties.add(property);
                }
                pcTemplatePropertyRepository.batchSave(properties, 1000);
                //推送进度60%
                processMessage(uuid, userId, total, 60, uuid, false);
                //保存账单属性值数据
                List<PcTemplateProperty> templateProperties = pcTemplatePropertyRepository.queryByTemplateId(template.getId());
                Map<String, PcTemplateProperty> map = Maps.newHashMap();
                for (PcTemplateProperty templateProperty : templateProperties) {
                    map.put(templateProperty.getName(), templateProperty);
                }
                List<PcTemplatePropertyValue> propertValues = Lists.newArrayList();
                //数据分组
                for (Integer key : data.keySet()) {
                    List<JSONObject> values = data.get(key);
                    String groupId = IDFactory.instance().uuid();
                    for (JSONObject value : values) {
                        String valueData = value.getString("value");
                        String name = value.getString("name");
                        Integer column = value.getInteger("column");
                        PcTemplatePropertyValue propertyValue = new PcTemplatePropertyValue();
                        propertyValue.setTemplateId(templateId);
                        propertyValue.setPropertyId(map.get(name).getId());
                        propertyValue.setPropertyValue(valueData);
                        propertyValue.setGroupId(groupId);
                        propertyValue.setLine(Long.valueOf(column));
                        propertyValue.setGmtCreate(new Date());
                        propertyValue.setCreatorId(userId);
                        propertyValue.setDeleted(false);
                        propertValues.add(propertyValue);
                    }
                }
                pcTemplatePropertyValueRepository.batchSave(propertValues, 1000);
                wb.write(baos);
                //推送进度100%
                processMessage(uuid, userId, total, 100, uuid, false);
            } catch (Exception e) {
                //数据回滚
                try {
                    pcBillTemplateRepository.delete(templateId, userId);
                    pcTemplatePropertyRepository.deleteByTemplateId(templateId, userId);
                    pcTemplatePropertyValueRepository.deleteByTemplateId(templateId, userId);
                } catch (Exception exception) {
                    logger.info("回滚数据失败", e);
                }
                processMessage(uuid, userId, 1, 0, uuid, false);
                logger.info("导入错误", e);
            } finally {
                if (baos != null) {
                    try {
                        baos.close();
                        wb.close();
                    } catch (IOException e) {
                        logger.info("关闭流错误", e);
                    }
                }
            }
        }
    }

    private void processMessage(String uuid, Long userId, int total, int success, String fileUuid, Boolean isSuccess) {
        Map<String, Object> messageMap = Maps.newHashMap();
        messageMap.put("total", total);
        if (!isSuccess) {
            messageMap.put("progress", 100);
            messageMap.put("success", 0);
            messageMap.put("fileName", fileUuid);
            messageMap.put("fail", 1);
        } else {
            messageMap.put("progress", success);
            if (100 == success) {
                messageMap.put("success", 1);
            } else {
                messageMap.put("success", 0);
            }
            messageMap.put("fileName", fileUuid);
            messageMap.put("fail", 0);
        }
        messageMap.put("uuid", uuid);
        RequestContext context = new RequestContext();
        context.setTarget(userId);
        context.setCatalog(Catalog.Manager);
        context.setTarget(0L);
        HashMap<String, Object> resultMap = Maps.newHashMap();
        resultMap.put("operate", "BILLIMPORT");
        resultMap.put("success", true);
        resultMap.put("context", context);
        resultMap.put("data", messageMap);
        resultMap.put("category", "bill");
        // 将执行结果写入mq 由web写入websock给前端
        resourceEventPublisher.send(resultMap);
    }


    /**
     * 统计数据量
     *
     * @param wb
     * @return
     */
    private int countDataNum(Workbook wb) {
        int total = 0;
        for (int n = 0; n < wb.getNumberOfSheets(); n++) {
            Sheet sheet = wb.getSheetAt(n);
            if (sheet.getSheetName().equals(ExcelHelper.EnumHelper.Schedulevalue.getValue())) {
                continue;
            }
            total += ExcelHelper.getRowNumberOfSheet(sheet);
        }
        return total;

    }

}